Package: NSTempRFA
Title: Adapts the Regional Frequency Analysis to Air Temperature
Version: 0.2.0
Authors@R: c(
    person("Gabriel Constantino", "Blain", , "gabriel.blain@sp.gov.br", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8832-7734")),
    person("Graciela R.", "Sobierajski", role = "aut",
           comment = c(ORCID = "0000-0002-7211-9268")),
    person("Leticia L.", "Martins", role = "aut",
           comment = c(ORCID = "0000-0002-0299-3005")),
    person("Adam H.", "Sparks", , "adamhsparks@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-0061-8359"))
  )
Description: Adapts the index-flood technique to extreme air temperature
    data. The package uses the additive approach proposed in Martins et
    al. (2022) <doi:10.1590/1678-4499.20220061> and adapt it to climate
    change conditions through the use of nonstationary parametric
    distributions.
License: MIT + file LICENSE
URL: https://github.com/gabrielblain/NSTempRFA
BugReports: https://github.com/gabrielblain/NSTempRFA/issues
Depends: R (>= 3.5)
Imports: extRemes, ismev, lmom, lmomRFA, MASS, Matrix, rrcov, spsUtil,
        stats, utils
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
Language: en-US
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-07-28 11:05:50 UTC; gabrielblain
Author: Gabriel Constantino Blain [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8832-7734>),
  Graciela R. Sobierajski [aut] (ORCID:
    <https://orcid.org/0000-0002-7211-9268>),
  Leticia L. Martins [aut] (ORCID:
    <https://orcid.org/0000-0002-0299-3005>),
  Adam H. Sparks [aut] (ORCID: <https://orcid.org/0000-0002-0061-8359>)
Maintainer: Gabriel Constantino Blain <gabriel.blain@sp.gov.br>
Repository: CRAN
Date/Publication: 2026-08-06 12:50:06 UTC

# Generated by roxygen2: do not edit by hand

export(Add_Discord)
export(Add_Heterogeneity)
export(Add_RegProb)
export(Add_RegQuant)
export(Best_model)
export(Dataset_add)
export(Fit_model)
export(Reg_par)
export(Reg_parCI)
export(Site_parCI)
importFrom(MASS,mvrnorm)
importFrom(Matrix,nearPD)
importFrom(extRemes,pevd)
importFrom(extRemes,qevd)
importFrom(ismev,gev.fit)
importFrom(lmom,pelkap)
importFrom(lmom,pelwak)
importFrom(lmom,quakap)
importFrom(lmom,quawak)
importFrom(lmomRFA,regsamlmu)
importFrom(rrcov,Cov)
importFrom(rrcov,getDistance)
importFrom(spsUtil,quiet)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,weighted.mean)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)

# NSTempRFA

<!-- badges: start -->

[![R-CMD-check](https://github.com/gabrielblain/NSTempRFA/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/gabrielblain/NSTempRFA/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

A package designed to apply the RFA technique to air temperature data
under climate change conditions.

## Installation

You can install the development version of NSTempRFA from GitHub with:

``` r
# install.packages("pak")
pak::pak("gabrielblain/NSTempRFA")
```

## Theoretical Background

The Regional Frequency Analysis (RFA; Dalrymple, 1960) technique has
been widely used to improve the probabilistic assessment of extreme
rainfall and flood events worldwide. The fundamental concept behind RFA
is to evaluate whether a group of time series from distinct sites can be
considered “acceptably homogeneous” (Hosking and Wallis, 1997). If this
condition is met, data from all sites can be pooled together,
effectively trading space for time, and thus extending the effective
data length (Smith et al. 2015).

In its original version, the RFA is valid only when the variable of
interest assumes strictly positive values. However, Martins et
al. (2022) verified that this technique can be effectively applied to
extreme maximum (Tmax) and minimum (Tmin) air temperature series, which
may assume both positive and negative values, by adopting the so-called
additive approach.

This additional approach is consistent with the physical characteristics
of temperature data. Since the fundamental temperature scale is the
Kelvin scale, where 0 K equals -273.15°C (or -459.67°F), both Tmax and
Tmin values fall within a relatively narrow range compared to their
distance from absolute zero. As a result, the variability in Tmax and
Tmin distributions can be treated as largely independent of their sample
mean (Martins et al. 2022). Moreover, when expressed in Celsius or
Fahrenheit, Tmin values may be negative, further justifying the use of
the additive approach.

Although the study by Martins et al. (2022) showed promising results, it
did not address the influence of long-term warming trends, which have
been documented in nearly all regions of the world (IPCC, 2021). This
calls for an extension of the RFA framework to incorporate
time-dependent, nonstationary statistical properties.

To overcome this limitation, Blain et al. (2026) propose the
Non-stationary Additive Regional Frequency Analysis, an extension of the
additive RFA that integrates nonstationary probabilistic models to
represent temporal changes in air temperature frequency distributions at
the regional scale. This package was designed to facilitate the
application of this new method to extreme Tmax and Tmin data under
nonstationary climate conditions.

## Basic Instructions

**Load the library for examples below**

``` r
library(NSTempRFA)
```

## Function `Add_Discord()`

### Description

This function calculates the Hosking and Wallis’ discordance measure
(discord) using the additive approach as described in Martins et
al. (2022) and Blain et al (2026).

### Usage

`Add_Discord(dataset)`

### Arguments

**dataset:** A numeric matrix with extreme air temperature data from
multiple sites. The first column must contain the years, and the
remaining columns contain temperature data from each site.

### Details

The discordance measures identify sites that are potentially discordant
within a regional frequency analysis framework for extreme air
temperature series.

### Value

A data.frame with seven columns: - `Local` Site identifier. -
`SampleSize` Number of observations for each site. - `l_1` L-moment
ratio (mean). - `l_2` L-moment ratio (L-scale). - `t_3` L-moment ratio
(L-skewness). - `t_4` L-moment ratio (L-kurtosis). - `t_5` L-moment
ratio (higher-order L-moment). - `discord` Original discordance
statistic indicating potential outlier status.

### Example

``` r
dataset <- TmaxCPC_SP
Add_Discord(dataset)
#>       Local SampleSize      l_1       l_2          t_3        t_4          t_5
#> 1   Pixel_1         34 35.49129 0.5702889 -0.046839615 0.12967963  0.117454055
#> 2   Pixel_2         34 33.47568 0.7114659  0.008514648 0.12875566 -0.016283412
#> 3   Pixel_3         34 34.42286 0.6216035 -0.006632936 0.11958897  0.050620060
#> 4   Pixel_4         34 35.05986 0.5818588  0.103225296 0.05659463  0.082984024
#> 5   Pixel_5         34 34.97852 0.7661857  0.180355170 0.09655880  0.101565708
#> 6   Pixel_6         34 34.54346 0.7651060  0.171180354 0.13719846  0.027045326
#> 7   Pixel_7         34 34.30666 0.7231769  0.114853094 0.16820816  0.015520001
#> 8   Pixel_8         34 32.81977 0.7120049  0.193897168 0.15569518  0.088295774
#> 9   Pixel_9         34 35.96037 0.8583378  0.186372622 0.10856159  0.011871484
#> 10 Pixel_10         34 35.46055 0.8180569  0.161493008 0.16103322 -0.009130147
#>      discord
#> 1  1.7860794
#> 2  1.6373754
#> 3  1.2644379
#> 4  2.4153036
#> 5  1.3050367
#> 6  0.7750693
#> 7  1.3517363
#> 8  1.9897006
#> 9  1.9723056
#> 10 1.3101471
```

## Function `Dataset_add()`

### Description

Subtract the sample mean of each column from their corresponding data.

### Usage

`dataset_add(dataset)`

### Arguments

**dataset:** A numeric matrix with extreme air temperature data from
multiple sites. The first column must contain the years. The remaining
columns contain temperature data from each site.

### Value

A list object with the following elements: - Subtracted_data: A matrix
of temperature values subtracted by their sample mean. - reg_mean:
Sample mean for each site.

### Example

``` r
dataset <- TmaxCPC_SP
Dataset_add(dataset)
#> $add_data
#>            Pixel_1     Pixel_2     Pixel_3     Pixel_4     Pixel_5     Pixel_6
#>  [1,]  0.806680901  0.39890895  0.51382457  0.07826973 -1.26789206 -0.62984085
#>  [2,] -1.276407019 -1.07100835 -1.46701802 -1.14710640 -0.92446249 -0.79555893
#>  [3,] -1.067941439 -0.36950841 -0.81885517 -1.31494545  0.54887659  0.35712051
#>  [4,] -0.876817479 -0.68805471 -0.57519520  0.28452660  0.66295130 -0.42699433
#>  [5,] -0.129327549 -0.74817052 -0.27107609  0.44226815 -0.18384283 -0.84130478
#>  [6,]  0.036894071 -0.63226476 -0.60400761 -0.79813026 -1.78046530 -0.92012787
#>  [7,]  2.638433681  1.95663676  2.08730327  1.33298088  0.14825327  0.86101532
#>  [8,]  0.444929351 -0.47622457  0.29207241  0.47764565 -1.57320898 -0.23893738
#>  [9,]  1.040842281 -1.64627423 -1.50838841 -1.25554298 -1.11890142 -1.59651947
#> [10,] -1.506883399 -2.42683187 -1.84099568 -0.21410774 -1.38093679 -1.37288666
#> [11,]  0.324857941 -1.55034795 -0.66855992  0.03188683 -1.41308706 -2.18091965
#> [12,]  0.371961821  0.58032069  0.53155910  1.13059594  0.68552286  1.42906951
#> [13,]  0.811525571  1.88877330  1.33351146  1.41313912  1.54653818  2.03365707
#> [14,] -1.943567049 -2.21049085 -1.74813450 -1.44470237 -1.37753027 -1.80634308
#> [15,]  0.049181211  1.02473674  0.53503811 -0.39466499 -1.64567678 -1.55134964
#> [16,]  0.021272881  1.69644389  1.78262340  1.20808388 -1.49557989 -0.52707673
#> [17,]  0.395166621  0.68113551  0.42061245 -0.31176399 -1.38004416 -1.31818772
#> [18,] -0.888894809  0.04522548 -0.37732685 -1.31680320 -0.47039526 -0.38025284
#> [19,]  0.380510551  0.32385859  0.34467708  0.10145928 -0.09685629  0.39632034
#> [20,]  1.573175651  0.94999156  0.47639858 -1.10231422  0.35593302  0.06438446
#> [21,]  0.407774201 -0.41990819 -0.17923725 -0.53943657 -0.17395132 -0.84939194
#> [22,]  0.859503021 -0.35633626  0.36359035  0.54243828  0.29317743  0.12120056
#> [23,] -0.951265109 -2.10568776 -1.96636570 -0.94400428 -0.74292105 -1.39243699
#> [24,] -0.411588449  0.17400584  0.12923061  0.67784096  1.06674463  0.70520019
#> [25,]  0.537821041  0.48855433  0.53112423  0.34025551  0.58240015  0.59571457
#> [26,]  0.567846521 -0.29345479  0.06583797 -0.45301078  0.07931025 -0.48486710
#> [27,] -1.640302429 -0.43832173 -1.15145481 -1.16273521  0.23186380 -0.02808381
#> [28,]  0.006403191 -0.80364766 -0.73622502 -0.89390205 -0.88955038 -0.80413437
#> [29,]  1.060312491  1.03521952  1.04153072  0.76687600  1.53927500  1.46383285
#> [30,]  0.902807461  2.65113673  1.84338772  1.91324593  3.89446527  3.31644058
#> [31,] -1.427316439  0.75615916  0.48415004  0.73680855  2.06313211  2.15882492
#> [32,] -1.255941169 -0.76440206 -0.90240467 -0.87110923 -0.38806265  0.15259933
#> [33,] -0.003717199  1.76204524  1.84878170  2.50021721  2.62044031  3.17444229
#> [34,]  0.142069091  0.58778224  0.18999111  0.18574120  1.98448069  1.31539154
#>           Pixel_7      Pixel_8     Pixel_9   Pixel_10
#>  [1,] -0.22739680  0.101871711 -1.17935888 -1.3090836
#>  [2,] -1.11911662 -1.011674659 -1.37078420 -1.4325768
#>  [3,] -0.36224635 -1.069068689  0.51408824  0.1935943
#>  [4,] -0.28177531 -0.048602829 -0.32252065 -0.1915604
#>  [5,] -0.37523539 -0.147311939 -0.98650304 -0.4959847
#>  [6,] -0.72715647 -1.013328329 -1.76890889 -0.2924286
#>  [7,]  0.91251104  0.823360671  0.64647349  0.7911132
#>  [8,]  0.33811681 -0.042125479 -1.63996068 -0.5049988
#>  [9,] -1.47085078 -0.268409509 -1.41883413 -1.5299050
#> [10,] -1.64861567 -1.330280079 -0.94290105 -0.8587967
#> [11,] -2.54513819 -1.010923159 -1.51050893 -1.5668694
#> [12,]  0.86088292  0.210121381  0.84732112  0.2927726
#> [13,]  1.25885122  0.920772781 -0.07001439  0.1628173
#> [14,] -1.92120631 -2.161268009 -1.39242498 -2.2887703
#> [15,] -0.35035212 -1.156675119 -2.66049520 -2.5558716
#> [16,]  1.23415487  0.213909371 -1.18843786 -1.3668496
#> [17,] -1.06052668 -0.898059619 -0.45487920 -0.5257050
#> [18,] -0.71214946 -1.450120699 -0.26275579 -0.2712456
#> [19,]  0.05151861 -1.352054369 -0.52853719 -1.0365235
#> [20,] -0.76660807 -1.295602579 -0.87979070 -0.8687760
#> [21,] -0.80171092 -1.016113059  0.43672999 -0.2468506
#> [22,]  0.12325780  0.496650921  1.04116114  1.2301925
#> [23,] -1.25236781 -0.008384479 -0.45061818 -0.5201851
#> [24,]  0.83279531  1.542698131  1.77628382  2.2546600
#> [25,]  0.62188833  0.806156381  0.65869578  1.1264862
#> [26,] -0.28351481  0.199932321  0.70248851  0.5215362
#> [27,] -0.52587779 -0.303710709  0.57683619  0.4834198
#> [28,] -0.75292094 -0.970168839 -0.89823095 -0.7556092
#> [29,]  1.14297597  0.548031081  2.41230067  1.0458702
#> [30,]  2.90345876  3.093185651  3.94813594  3.3613800
#> [31,]  2.03954809  2.267257911  2.89697703  2.9804519
#> [32,]  0.48710744  0.231853711 -1.19216481 -0.7576615
#> [33,]  3.36168020  3.492275461  2.49659404  2.9711364
#> [34,]  1.01601903  1.605804671  2.16454371  1.9608215
#> attr(,"scaled:center")
#>  Pixel_1  Pixel_2  Pixel_3  Pixel_4  Pixel_5  Pixel_6  Pixel_7  Pixel_8 
#> 35.49129 33.47568 34.42286 35.05986 34.97852 34.54346 34.30666 32.81977 
#>  Pixel_9 Pixel_10 
#> 35.96037 35.46055 
#> 
#> $reg_mean
#>  Pixel_1  Pixel_2  Pixel_3  Pixel_4  Pixel_5  Pixel_6  Pixel_7  Pixel_8 
#> 35.49129 33.47568 34.42286 35.05986 34.97852 34.54346 34.30666 32.81977 
#>  Pixel_9 Pixel_10 
#> 35.96037 35.46055
```

## Function `Add_Heterogeneity()`

### Description

Calculates Hosking and Wallis’ heterogeneity measure using the additive
approach as described in Martins et al. (2022) and Blain et al (2026).

### Usage

Add_Heterogeneity(dataset.add, rho, Ns)

### Arguments

**dataset.add**: A matrix of temperature values subtracted by their
sample mean, as calculated by the `Add_Discord()` function. **rho**: A
single numeric value (constant) describing the average correlation among
the sites. It must be larger than -1.0 and lower than 1.0. **Ns**:
Number of simulated groups of series. Default is 100, but at least 500
is recommended.

### Example

``` r
rho <- 0.51
Ns <- 500
add.data <- Dataset_add(TmaxCPC_SP)
Add_Heterogeneity(dataset.add = add.data$add_data, rho = rho, Ns = Ns)
#> [1] 0.8951683
```

## Function `Best_model()`

### Description

Calculates the time-varying parameters of the best fitting GEV model for
each site. See **Methodological Details** for more information on the
candidate GEV-models considered in this package.

### Usage

`Best_model(add_data)`

### Arguments

**add_data**: A matrix of temperature values subtracted by their sample
mean. May be generated by the `dataset_add()` function

### Value

A list object with the following elements: - `best`: The best model
among the six candidates. - `atsite.models`: The time-varying parameters
of the best model.

### Example

``` r
add.data <- Dataset_add(TmaxCPC_SP)
Best_model(add.data = add.data$add_data)
#> $best
#> [1] 2
#> 
#> $atsite.models
#>           mu0          mu1    sigma0 sigma1       shape size
#> 1  -0.2864336 -0.004971809 0.9852760      0 -0.23987862   34
#> 2  -1.0227894  0.035332791 1.1698222      0 -0.29729970   34
#> 3  -0.7748006  0.022975995 1.0269278      0 -0.26978834   34
#> 4  -0.6901442  0.015431062 0.8563226      0 -0.11111473   34
#> 5  -1.6362737  0.062523525 0.8888875      0  0.02677818   34
#> 6  -1.5547575  0.059680647 1.0063711      0 -0.08750300   34
#> 7  -1.5262793  0.063459052 1.0688034      0 -0.23920558   34
#> 8  -1.4567797  0.057640734 1.0035459      0 -0.15821851   34
#> 9  -2.0731388  0.094374300 1.1884099      0 -0.27779739   34
#> 10 -2.0294572  0.096551734 1.2348429      0 -0.41420337   34
```

## Function `Reg_par()`

### Description

Calculates the regional parameters for the regional time-varying GEV
distribution.

### Usage

`Reg_par(best_model)`

### Arguments

**best_model**: A 7-column data.frame as that generated by
`Best_model()` function. - 1st column is the mu0 parameters - 2nd is the
mu1 parameters - 3rd is the mu2 parameters - 4th is the sigma0
parameters - 5th is the sigma1 parameters - 6th is the shape
parameters - 7th is the sample sizes.

### Value

A `data.frame` with the regional parameters of the regional time-varying
distribution.

### Example

``` r
add.data <- Dataset_add(TmaxCPC_SP)
best.parms <- Best_model(add.data = add.data$add_data)
Reg_par(best_model = best.parms$atsite.models)
#>   weighted_mu0 weighted_mu1 weighted_sigma0 weighted_sigma1 weighted_shape
#> 1    -1.305085    0.0502998        1.042921               0     -0.2068231
```

## Function `Reg_parCI()`

### Description

Calculates the 95% confidence intervals of time-varying parameters of
the regional GEV distribution. It follows the method proposed by Burn
(2003) and O’Brien and Burn (2014). The spatial dependence between sites
is preserved.

### Usage

`Reg_parCI(add_data, reg_par, reg_mean, max_time, n.boots)`

### Arguments

**add_data**: A matrix of temperature values subtracted by their sample
mean. May be generated by the `dataset_add()` function. **reg_par**: A
6-column and 1-row data.frame as that generated by `reg_par()`, that
is: - 1st column is the mu0 parameter - 2nd is the mu1 parameter - 3rd
is the mu2 parameter - 4th is the sigma0 parameter - 5th is the sigma1
parameter - 6th is the shape parameter **reg_mean**: A numeric vector of
site mean temperatures as returned by `Dataset_add()$reg_mean`. Used to
restore bootstrap replicates from centered to original scale before
refitting. Must have length equal to the number of columns in
`add_data`. **max_time**: A single number describing the number of the
year that the time-varying parameters should be calculated. For example,
if the users need to calculated the parameters for the first year
max_time is set to 1 and the 30th year max_time is set to 30.
**n.boots**: A single number describing the number of copies of the
original dataset. Whenever possible n.boots should be set to 999 (its
default value), as suggested by Burn (2003) and O’Brien and Burn (2014).

### Value

A matrix containing the 95% confidence intervals (lower and upper
bounds) of the time-varying parameter estimates.

### Example

``` r
add.data <- Dataset_add(TmaxCPC_SP)
best.parms <- Best_model(add.data = add.data$add_data)
regional.parms <- Reg_par(best_model = best.parms$atsite.models)
Reg_parCI(
  add_data = add.data$add_data,
  model = 2,
  reg_par = regional.parms,
  n.boots = 100
)
#> This calculation may take some time.
#>              weighted_mu0 weighted_mu1 weighted_sigma0 weighted_sigma1
#> Lower 95% CI   -1.7947762   0.02394311       0.8346277               0
#> Upper 95% CI   -0.7570151   0.07383739       1.2716319               0
#>              weighted_shape
#> Lower 95% CI   -0.409317501
#> Upper 95% CI   -0.006443904
```

## Function `Site_parCI`

### Description

Calculates the 95% confidence intervals of time-varying parameters of a
at-site GEV distribution. It follows the method proposed by Burn (2003).

### Usage

`Site_parCI(data_site, site_par, max_time, n.boots)`

### Arguments

**data_site**: A vector or single column matrix of temperature values
subtracted by its sample mean. **site_par**: A 6-column and 1-row
data.frame; it may be obtained from `best.parms()`. • 1st column is the
mu0 parameter • 2nd is the mu1 parameter • 3rd is the mu2 parameter •
4th is the sigma0 parameter • 5th is the sigma1 parameter • 6th is the
shape parameter **max_time**: A single number describing the number of
the year that the time-varying parameters should be calculated. For
example, if the users need to calculated the parameters for the first
year max_time is set to 1 and the 30th year max_time is set to 30.
**n.boots**: A single number describing the number of copies of the
original data sample. Whenever possible n.boots should be set to 999
(its default value), as suggested by Burn (2003).

### Value

A matrix containing the 95% confidence intervals of the time-varying
parameter estimates for a at-site model.

### Example

``` r
temperatures <- TmaxCPC_SP$Pixel_1
model <- 2
site_par <- Fit_model(temperatures, model)
Site_parCI(
  atsite_temp = temperatures,
  model = model,
  site_par = site_par[1:5],
  n.boots = 100
)
#> This calculation may take some time.
#>                   mu0         mu1    sigma0 sigma1       shape
#> Lower 95% CI 34.60627 -0.03872880 0.7932048      0 -1.03961985
#> Upper 95% CI 35.97520  0.02436096 1.4470887      0 -0.04433179
```

## Function `Add_RegProb`

### Description

Calculates the cumulative probability of extreme air temperature data
using the Additional nonstationary RFA approach based on the GEV
distribution.

### Usage

`Add_RegProb(quantiles, reg_par, site_mean, max_time)`

### Arguments

**quantiles**: A numeric vector with no missing data with extreme air
temperatures. **reg_par**: A 6-column and 1-row data.frame as that
generated by reg_par(), that is: - 1st column is the mu0 parameter - 2nd
is the mu1 parameter - 3rd is the mu2 parameter - 4th is the sigma0
parameter - 5th is the sigma1 parameter - 6th is the shape parameter

**site_mean**: A single number with site’s average value for the air
temperature data. **max_time**: A single number describing the number of
the year that the time-varying parameters should be calculated. For
example, if the users need to calculated the parameters for the first
year `max_time` is set to 1 and the 30th year max_time is set to 30.

### Value

Cumulative probabilities of extreme air temperature data calculated from
the Additional nonstationary RFA approach.

### Example

``` r
 quantiles <- c(37.2, 37.5, 37.8, 37.9, 38.0, 38.1, 38.2, 38.5, 39.0)
 add.data <- Dataset_add(TmaxCPC_SP)
 best.parms <- Best_model(add.data = add.data$add_data)
 regional_pars <- Reg_par(best_model = best.parms$atsite.models)
 Add_RegProb(
   quantiles = quantiles,
   regional_pars = regional_pars,
   site_temp = TmaxCPC_SP$Pixel_1,
   n.year = 34
 )
#>            [,1]
#>  [1,] 0.7902025
#>  [2,] 0.8545796
#>  [3,] 0.9038670
#>  [4,] 0.9172022
#>  [5,] 0.9291197
#>  [6,] 0.9397071
#>  [7,] 0.9490556
#>  [8,] 0.9706002
#>  [9,] 0.9902066
```

## Function `Add_RegQuant`

### Description

Calculates quantiles estimates from cumulative probabilities of extreme
air temperature data using the Additional nonstationary RFA approach
based on the GEV distribution.

### Usage

`Add_RegQuant(prob, reg_par, site_mean, max_time)`

### Arguments

**prob**: A numeric vector with no missing data with the cumulative
probabilities (between 0 and 1). **reg_par**: A 6-column and 1-row
data.frame as that generated by `reg_par()`, that is: - 1st column is
the mu0 parameter - 2nd is the mu1 parameter - 3rd is the mu2
parameter - 4th is the sigma0 parameter - 5th is the sigma1 parameter -
6th is the shape parameter

**site_mean**: A single number with site’s average value for the air
temperature data **max_time**: A single number describing the number of
the year that the time-varying parameters should be calculated. For
example, if the users need to calculated the parameters for the first
year `max_time` is set to 1 and the 30th year `max_time` is set to 30.

### Value

Quantile estimates of extreme air temperature data calculated from the
Additional nonstationary RFA approach.

### Example

``` r
 prob <- c(0.8, 0.85, 0.90, 0.92, 0.93, 0.94, 0.95, 0.97, 0.99)
 add.data <- Dataset_add(TmaxCPC_SP)
 best.parms <- Best_model(add.data = add.data$add_data)
 regional_pars <- Reg_par(best_model = best.parms$atsite.models)
 Add_RegQuant(
   prob = prob,
   regional_pars = regional_pars,
   site_temp = TmaxCPC_SP$Pixel_1,
   n.year = 34
 )
#>      Q80      Q85      Q90      Q92      Q93      Q94      Q95      Q97 
#> 37.24134 37.47600 37.77291 37.92247 38.00787 38.10295 38.21086 38.48963 
#>      Q99 
#> 38.99157
```

## BugReports:

<https://github.com/gabrielblain/NSTempRFA/issues>

## License:

MIT

## Authors:

Gabriel Constantino Blain, Graciela da Rocha Sobierajski, Leticia Lopes
Martins. Maintainer: Gabriel Constantino Blain,
<gabriel.blain@sp.gov.br>

## Acknowledgments:

## References

Blain, G. C., Sobierajski, G. R. and Martins, L. L. (2026). The
non-stationary additive regional frequency analysis method. Theoretical
and Applied Climatology, 157, 157-219. DOI: 10.1007/s00704-026-06148-4.

Burn, D.H., 2003. The use of resampling for estimating confidence
intervals for single site and pooled frequency analysis. Hydrol. Sci. J.
48 (1), 25–38. DOI: 10.1623/hysj.48.1.25.43485.

O’Brien, N.L., Burn,H.D. A nonstationary index-flood technique for
estimating extreme quantiles for annual maximum streamflow, Journal of
Hydrology, v.519, 2014. DOI: 10.1016/j.jhydrol.2014.09.041.

Dalrymple, T. (1960). Flood frequency analysis. Geological Survive Water
Supply Paper, 1543-A, 11-51. DOI: 10.3133/wsp1543A.

\[IPCC\] Intergovernmental Panel on Climate Change (2021). Climate
Change 2021: The Physical Science Basis. Contribution of Working Group I
to the Sixth Assessment Report of the Intergovernmental Panel on Climate
Change. Cambridge University Press. Available at:
<https://www.ipcc.ch/report/ar6/wg1/> Accessed on: Jan. 1, 2025.

Martins, L. L., Souza, J. C., Sobierajski, G. R. and Blain, G. C.
(2022). Is it possible to apply the regional frequency analysis to daily
extreme air temperature data? Bragantia, 81, 1-22. DOI:
10.1590/1678-4499.20220061.

Smith, A., Sampson, C., & Bates, P. D. (2015). Regional flood frequency
analysis at the global scale. Water Resources Research, 51(1), 539–553.
DOI: 10.1002/2014WR015814.

Hosking JRM, Wallis JR (1997) Regional frequency analysis: an approach
based on L-moments. Cambridge University Press. DOI:
10.1017/cbo9780511529443.

#' Add_Heterogeneity
#'
#' @param dataset.add
#' A numeric matrix of air temperature data as calculated by
#' `Dataset_add()`.
#' @param rho
#' A single numeric value describing the average inter-site correlation.
#' Must be strictly between -1 and 1.
#' @param Ns
#' Number of simulated groups of series. Default is 100; at least 500
#' is recommended.
#'
#' @returns
#' Hosking and Wallis' heterogeneity measure using an additive approach,
#' as proposed in Martins et al. (2022)
#' \doi{10.1590/1678-4499.20220061}.
#' Returns `NA` if the regional distribution cannot be fitted or if
#' simulated statistics have zero variance.
#'
#' @export
#' @importFrom lmomRFA regsamlmu
#' @importFrom lmom pelkap pelwak quakap quawak
#' @importFrom MASS mvrnorm
#' @importFrom stats pnorm sd
#' @importFrom Matrix nearPD
#'
#' @examples
#' add.data <- Dataset_add(TmaxCPC_SP)
#' Add_Heterogeneity(dataset.add = add.data$add_data, rho = 0.51, Ns = 500)
Add_Heterogeneity <- function(dataset.add, rho, Ns) {
  dataset.add <- check_heterogeneity_data(dataset.add) # input_checks.R
  check_rho(rho) # input_checks.R
  check_Ns(Ns) # input_checks.R

  n.sites <- ncol(dataset.add)
  x1.atoutset <- lmomRFA::regsamlmu(dataset.add, lcv = FALSE)
  weight <- sum(x1.atoutset[, 2])
  site_years <- x1.atoutset[, 2]
  max.n.years <- max(site_years)

  wt_mean <- function(col) sum(x1.atoutset[, 2] * x1.atoutset[, col]) / weight
  rmom <- c(0, wt_mean(4), wt_mean(5), wt_mean(6), wt_mean(7))
  reg.l2 <- rmom[2]

  reg.par <- fit_regional_dist(rmom)

  if (is.null(reg.par)) {
    warning("Failed to fit regional distribution. Returning NA.", call. = FALSE)
    return(NA)
  }

  V <- sqrt(sum(x1.atoutset[, 2] * (x1.atoutset[, 4] - reg.l2)^2) / weight)

  sigma <- make_sigma(rho, n.sites)

  V.sim <- vapply(
    seq_len(Ns),
    simulate_V,
    FUN.VALUE = numeric(1),
    sigma = sigma,
    n.sites = n.sites,
    site_years = site_years,
    weight = weight,
    reg.l2 = reg.l2,
    x1.atoutset = x1.atoutset,
    reg.par = reg.par,
    max.n.years = max.n.years
  )

  return(compute_H(V, V.sim))
}


# -----------------------------------------------------------------------------
# Internal: build and repair the inter-site correlation matrix.
# -----------------------------------------------------------------------------
#' @noRd
make_sigma <- function(rho, n.sites) {
  sigma <- matrix(rho, n.sites, n.sites)
  diag(sigma) <- 1

  if (any(eigen(sigma, only.values = TRUE)$values <= 0)) {
    sigma <- as.matrix(Matrix::nearPD(sigma, corr = TRUE)$mat)
  }

  return(sigma)
}


# -----------------------------------------------------------------------------
# Internal: compute H from observed V and simulated V.sim,
# warning and returning NA if V.sim is degenerate.
# -----------------------------------------------------------------------------
#' @noRd
compute_H <- function(V, V.sim) {
  V.sim_sd <- stats::sd(V.sim, na.rm = TRUE)

  if (!all(is.finite(V.sim)) || V.sim_sd == 0) {
    warning(
      "V.sim contains invalid values or zero variance. Returning NA.",
      call. = FALSE
    )
    return(NA)
  }

  return((V - mean(V.sim, na.rm = TRUE)) / V.sim_sd)
}


# -----------------------------------------------------------------------------
# Internal: fit regional distribution, trying kappa then Wakeby as fallback.
# Returns the parameter vector or NULL on failure.
# -----------------------------------------------------------------------------
#' @noRd
fit_regional_dist <- function(rmom) {
  reg.par <- try(lmom::pelkap(rmom), silent = TRUE)

  if (inherits(reg.par, "try-error") || !all(is.finite(reg.par))) {
    reg.par <- try(lmom::pelwak(rmom), silent = TRUE)
  }

  if (inherits(reg.par, "try-error") || !all(is.finite(reg.par))) {
    return(NULL)
  }

  return(reg.par)
}


# -----------------------------------------------------------------------------
# Internal: quantile function dispatcher — Wakeby (length 5) or kappa.
# -----------------------------------------------------------------------------
#' @noRd
regional_quantile <- function(u, reg.par) {
  if (length(reg.par) == 5) {
    return(lmom::quawak(u, reg.par))
  } else {
    return(lmom::quakap(u, reg.par))
  }
}


# -----------------------------------------------------------------------------
# Internal: simulate one V statistic from the fitted regional distribution.
# Returns NA_real_ if mvrnorm fails or produces non-finite values.
# -----------------------------------------------------------------------------
#' @noRd
simulate_V <- function(
  ns,
  sigma,
  n.sites,
  site_years,
  weight,
  reg.l2,
  x1.atoutset,
  reg.par,
  max.n.years
) {
  u.sim <- tryCatch(
    stats::pnorm(
      MASS::mvrnorm(n = max.n.years, mu = rep(0, n.sites), Sigma = sigma)
    ),
    error = function(e) {
      warning(
        "mvrnorm failed (simulation ",
        ns,
        "): ",
        conditionMessage(e),
        call. = FALSE
      )
      NULL
    }
  )

  if (is.null(u.sim) || anyNA(u.sim)) {
    return(NA_real_)
  }

  # Simulate data for each site and pad to max.n.years rows
  data.sim <- do.call(
    cbind,
    mapply(
      function(yrs, col) {
        out <- rep(NA_real_, max.n.years)
        out[seq_len(yrs)] <- regional_quantile(
          u.sim[seq_len(yrs), col],
          reg.par
        )
        out
      },
      site_years,
      seq_len(n.sites),
      SIMPLIFY = FALSE
    )
  )

  x1.sim <- lmomRFA::regsamlmu(data.sim, lcv = FALSE)
  reg.l2.sim <- sum(x1.atoutset[, 2] * x1.sim[, 4]) / weight
  numerator <- x1.atoutset[, 2] * (x1.sim[, 4] - reg.l2.sim)^2

  return(sqrt(sum(numerator) / weight))
}

test_that("Add_Heterogeneity returns a numeric H value", {
  set.seed(123)

  dataset.add <- matrix(rnorm(1500), ncol = 15)

  result <- Add_Heterogeneity(
    dataset.add = dataset.add,
    rho = 0.5,
    Ns = 100
  )

  expect_length(result, 1)
  expect_type(result, "double")
  expect_false(is.na(result))
  expect_true(is.finite(result))
})


test_that("Add_Heterogeneity accepts the minimum number of sites", {
  set.seed(123)

  dataset.add <- matrix(rnorm(300), ncol = 3)

  result <- Add_Heterogeneity(
    dataset.add,
    rho = 0.5,
    Ns = 100
  )

  expect_type(result, "double")
  expect_length(result, 1)
})


test_that("Add_Heterogeneity throws error when fewer than 3 sites are supplied", {
  dataset.add <- matrix(rnorm(300), ncol = 2)

  expect_error(
    Add_Heterogeneity(dataset.add, rho = 0.5, Ns = 100),
    "The number of sites should be equal to or larger than 3.",
    fixed = TRUE
  )
})


test_that("Add_Heterogeneity throws error for sites with less than 10 years", {
  dataset.add <- matrix(rnorm(1500), ncol = 15)

  dataset.add[1:95, 1] <- NA

  expect_error(
    Add_Heterogeneity(dataset.add, rho = 0.5, Ns = 100),
    "All sites must have at least 10 observations."
  )
})


test_that("Add_Heterogeneity accepts exactly 10 observations", {
  set.seed(123)

  dataset.add <- matrix(rnorm(150), nrow = 10, ncol = 15)

  result <- Add_Heterogeneity(
    dataset.add,
    rho = 0.5,
    Ns = 100
  )

  expect_type(result, "double")
})


test_that("Add_Heterogeneity throws error if Ns is too small", {
  dataset.add <- matrix(rnorm(1500), ncol = 15)

  expect_error(
    Add_Heterogeneity(dataset.add, rho = 0.5, Ns = 99),
    "Ns should be larger than 99"
  )
})


test_that("Add_Heterogeneity accepts Ns equal to 100", {
  set.seed(123)

  dataset.add <- matrix(rnorm(1500), ncol = 15)

  result <- Add_Heterogeneity(
    dataset.add,
    rho = 0.5,
    Ns = 100
  )

  expect_type(result, "double")
})


test_that("Add_Heterogeneity accepts negative correlations", {
  set.seed(123)

  dataset.add <- matrix(rnorm(1500), ncol = 15)

  result <- Add_Heterogeneity(
    dataset.add,
    rho = -0.5,
    Ns = 100
  )

  expect_type(result, "double")
  expect_true(is.finite(result))
})


test_that("Add_Heterogeneity accepts correlations close to one", {
  set.seed(123)

  dataset.add <- matrix(rnorm(1500), ncol = 15)

  result <- Add_Heterogeneity(
    dataset.add,
    rho = 0.99,
    Ns = 100
  )

  expect_type(result, "double")
})


test_that("Add_Heterogeneity rejects rho greater than one", {
  dataset.add <- matrix(rnorm(1500), ncol = 15)

  expect_error(
    Add_Heterogeneity(dataset.add, rho = 1.1, Ns = 100),
    "'rho' must be a single numeric value strictly between -1 and 1."
  )
})


test_that("Add_Heterogeneity rejects rho smaller than minus one", {
  dataset.add <- matrix(rnorm(1500), ncol = 15)

  expect_error(
    Add_Heterogeneity(dataset.add, rho = -1.1, Ns = 100),
    "'rho' must be a single numeric value strictly between -1 and 1."
  )
})


test_that("Add_Heterogeneity rejects rho equal to one", {
  dataset.add <- matrix(rnorm(1500), ncol = 15)

  expect_error(
    Add_Heterogeneity(dataset.add, rho = 1, Ns = 100),
    "'rho' must be a single numeric value strictly between -1 and 1."
  )
})


test_that("Add_Heterogeneity rejects rho equal to minus one", {
  dataset.add <- matrix(rnorm(1500), ncol = 15)

  expect_error(
    Add_Heterogeneity(dataset.add, rho = -1, Ns = 100),
    "'rho' must be a single numeric value strictly between -1 and 1."
  )
})


test_that("Add_Heterogeneity rejects rho vectors", {
  dataset.add <- matrix(rnorm(1500), ncol = 15)

  expect_error(
    Add_Heterogeneity(dataset.add, rho = c(0.2, 0.5), Ns = 100),
    "'rho' must be a single numeric value strictly between -1 and 1."
  )
})


test_that("Add_Heterogeneity rejects non-numeric rho", {
  dataset.add <- matrix(rnorm(1500), ncol = 15)

  expect_error(
    Add_Heterogeneity(dataset.add, rho = "0.5", Ns = 100),
    "'rho' must be a single numeric value strictly between -1 and 1."
  )
})


test_that("Add_Heterogeneity returns a finite value", {
  set.seed(123)

  dataset.add <- matrix(rnorm(1500), ncol = 15)

  result <- Add_Heterogeneity(
    dataset.add,
    rho = 0.5,
    Ns = 100
  )

  expect_true(is.finite(result))
})


test_that("Add_Heterogeneity returns a scalar", {
  set.seed(123)

  dataset.add <- matrix(rnorm(1500), ncol = 15)

  result <- Add_Heterogeneity(
    dataset.add,
    rho = 0.5,
    Ns = 100
  )

  expect_length(result, 1)
})

test_that("Add_Heterogeneity falls back to Wakeby when Kappa fails", {
  local_mocked_bindings(
    pelkap = function(...) NA,
    .package = "NSTempRFA"
  )

  set.seed(123)

  dataset.add <- matrix(rnorm(1500), ncol = 15)

  expect_error(
    Add_Heterogeneity(dataset.add, rho = 0.5, Ns = 100),
    NA
  )
})

test_that("Add_Heterogeneity returns NA when both regional distributions fail", {
  local_mocked_bindings(
    pelkap = function(...) NA,
    pelwak = function(...) NA,
    .package = "lmom" # <-- mock in lmom, not NSTempRFA
  )

  set.seed(123)

  dataset.add <- matrix(rnorm(1500), ncol = 15)

  expect_warning(
    result <- Add_Heterogeneity(
      dataset.add,
      rho = 0.5,
      Ns = 100
    ),

    "Failed to fit regional distribution"
  )

  expect_true(is.na(result))
})

test_that("Add_Heterogeneity returns NA when V.sim has zero variance", {
  fake_lmom <- matrix(
    c(
      1,
      100,
      0,
      1,
      0.1,
      0.2,
      0.3,
      2,
      100,
      0,
      1,
      0.1,
      0.2,
      0.3,
      3,
      100,
      0,
      1,
      0.1,
      0.2,
      0.3
    ),
    nrow = 3,
    byrow = TRUE
  )

  local_mocked_bindings(
    regsamlmu = function(...) fake_lmom,
    .package = "lmomRFA" # <-- mock in lmomRFA, not NSTempRFA
  )
  dataset.add <- matrix(rnorm(300), ncol = 3)

  expect_warning(
    result <- Add_Heterogeneity(
      dataset.add,
      rho = 0.5,
      Ns = 100
    ),

    "V.sim contains invalid values"
  )

  expect_true(is.na(result))
})

test_that("Add_Heterogeneity repairs non positive definite correlation matrices", {
  set.seed(123)

  dataset.add <- matrix(rnorm(1500), ncol = 15)

  result <- Add_Heterogeneity(
    dataset.add,
    rho = -0.9,
    Ns = 100
  )

  expect_type(result, "double")
  expect_true(is.finite(result))
})

#' Confidence intervals for site-specific GEV parameters
#'
#' @name Site_parCI
#' @param atsite_temp
#' A numeric vector or single-column matrix of air temperature data.
#' @param model
#' A single integer from 1 to 4 defining the GEV model.
#' @param site_par
#' A 1-row, 5-column data.frame or matrix generated by Fit_model().
#' @param n.boots
#' Number of bootstrap replicates. Values smaller than 100 are not allowed.
#' Defaults to `999`.
#' @param progress
#' Logical scalar controlling whether a progress bar is displayed.
#' `TRUE` forces the bar on, `FALSE` forces it off, and
#' `NULL` (default) defers to `getOption("NSTempRFA.progress")`,
#' which itself falls back to `interactive()`. Set
#' `options(NSTempRFA.progress = FALSE)` to suppress the bar globally.
#'
#' @returns
#' A matrix containing the lower and upper 95% confidence limits of
#' the estimated parameters.
#'
#' @export
#' @importFrom stats quantile
#' @importFrom utils txtProgressBar setTxtProgressBar
#'
#' @examplesIf interactive()
#' temperatures <- TmaxCPC_SP$Pixel_1
#' model <- 2
#' site_par <- Fit_model(temperatures, model)
#'
#' Site_parCI(
#'   atsite_temp = temperatures,
#'   model       = model,
#'   site_par    = site_par[1, 1:5],
#'   n.boots     = 100
#' )
Site_parCI <- function(
  atsite_temp,
  model,
  site_par,
  n.boots = 999,
  progress = NULL
) {
  #----------------------------------------------------------
  # Input checks
  #----------------------------------------------------------
  check_model(model)
  check_n.boots(n.boots)
  data_site <- check_atsite_temp(atsite_temp)
  site_par <- check_site_par(site_par)
  max_time <- length(data_site)

  #----------------------------------------------------------
  # Time-varying parameters
  #----------------------------------------------------------
  time <- seq_len(max_time)
  par.temporal <- matrix(NA_real_, max_time, 3)
  par.temporal[, 1] <- site_par[1] + site_par[2] * time
  par.temporal[, 2] <- site_par[3] + site_par[4] * time
  par.temporal[, 3] <- site_par[5]

  if (any(par.temporal[, 2] <= 0)) {
    stop("The scale parameter becomes non-positive over time.", call. = FALSE)
  }

  #----------------------------------------------------------
  # Transform to Gumbel space
  #----------------------------------------------------------
  use_gumbel <- abs(site_par[5]) < sqrt(.Machine$double.eps)

  if (use_gumbel) {
    IDD.series <- (data_site - par.temporal[, 1]) / par.temporal[, 2]
  } else {
    z <- 1 +
      par.temporal[, 3] *
        ((data_site - par.temporal[, 1]) / par.temporal[, 2])
    if (any(z <= 0, na.rm = TRUE)) {
      stop("Invalid transformed values encountered.", call. = FALSE)
    }
    IDD.series <- (1 / par.temporal[, 3]) * log(z)
  }

  #----------------------------------------------------------
  # Pre-generate bootstrap indices and back-transformation constants
  #----------------------------------------------------------
  boot_indices <- matrix(
    sample.int(max_time, size = max_time * n.boots, replace = TRUE),
    nrow = max_time,
    ncol = n.boots
  )

  mu_vec <- par.temporal[, 1]
  sigma_vec <- par.temporal[, 2]
  xi <- site_par[5]

  site_par.overall.boot <- matrix(NA_real_, nrow = n.boots, ncol = 5)

  #----------------------------------------------------------
  # Progress bar — controlled by use_progress_bar()
  #----------------------------------------------------------
  message("This calculation may take some time.")
  show_pb <- use_progress_bar(progress)
  if (show_pb) {
    pb <- utils::txtProgressBar(min = 0, max = n.boots, style = 3)
  }

  #----------------------------------------------------------
  # Bootstrap loop
  #----------------------------------------------------------
  for (r in seq_len(n.boots)) {
    resampled_IDD <- IDD.series[boot_indices[, r]]

    if (use_gumbel) {
      back.orig <- mu_vec + sigma_vec * resampled_IDD
    } else {
      back.orig <- mu_vec + (sigma_vec / xi) * (exp(resampled_IDD * xi) - 1)
    }

    parameters <- Fit_model(temperatures = back.orig, model = model)
    site_par.overall.boot[r, ] <- as.numeric(parameters[1, 1:5, drop = FALSE])

    if (show_pb) utils::setTxtProgressBar(pb, r)
  }

  if (show_pb) {
    close(pb)
  }

  #----------------------------------------------------------
  # Confidence intervals
  #----------------------------------------------------------
  CI_lower <- apply(
    site_par.overall.boot,
    2,
    quantile,
    probs = 0.025,
    na.rm = TRUE
  )
  CI_upper <- apply(
    site_par.overall.boot,
    2,
    quantile,
    probs = 0.975,
    na.rm = TRUE
  )

  CI_matrix <- rbind(CI_lower, CI_upper)
  rownames(CI_matrix) <- c("Lower 95% CI", "Upper 95% CI")
  colnames(CI_matrix) <- c("mu0", "mu1", "sigma0", "sigma1", "shape")
  CI_matrix
}

test_that("Site_parCI returns a matrix with the correct dimensions", {
  set.seed(123)
  temperatures <- rnorm(500) # large sample for reliable convergence
  model <- 2
  site_par <- Fit_model(temperatures, model)

  skip_if(
    anyNA(site_par[1, 1:5]),
    "Fit_model did not converge — skipping"
  )

  result <- Site_parCI(
    atsite_temp = temperatures,
    model = model,
    site_par = site_par[1, 1:5],
    n.boots = 100
  )

  expect_true(is.matrix(result))
  expect_identical(dim(result), c(2L, 5L))
  expect_named(
    as.data.frame(result),
    c("mu0", "mu1", "sigma0", "sigma1", "shape")
  )
  expect_true(all(is.finite(result[, c(1, 3, 5)])))
})


test_that("Site_parCI accepts a single-column matrix", {
  set.seed(123)

  atsite_temp <- matrix(rnorm(50), ncol = 1)
  site_par <- matrix(c(0, 0.02, 1, 0.001, 0.1), nrow = 1)

  result <- Site_parCI(
    atsite_temp = atsite_temp,
    model = 1,
    site_par = site_par,
    n.boots = 100
  )

  expect_true(is.matrix(result))
})


test_that("Site_parCI rejects invalid model values", {
  temp <- rnorm(50)
  pars <- matrix(c(0, 0, 1, 0, 0.1), nrow = 1)

  expect_error(
    Site_parCI(temp, 0, pars, 100),
    "`model` must be a single integer"
  )

  expect_error(
    Site_parCI(temp, 5, pars, 100),
    "`model` must be a single integer"
  )

  expect_error(
    Site_parCI(temp, "two", pars, 100),
    "`model` must be a single integer"
  )
})


test_that("Site_parCI rejects invalid n.boots", {
  temp <- rnorm(50)
  pars <- matrix(c(0, 0, 1, 0, 0.1), nrow = 1)

  expect_error(
    Site_parCI(temp, 1, pars, 50),
    "`n.boots`"
  )

  expect_error(
    Site_parCI(temp, 1, pars, 99.5),
    "`n.boots`"
  )
})


test_that("Site_parCI rejects series shorter than 10 observations", {
  temp <- rnorm(9)
  pars <- matrix(c(0, 0, 1, 0, 0.1), nrow = 1)

  expect_error(
    Site_parCI(temp, 1, pars, 100),
    "at least 10 years"
  )
})


test_that("Site_parCI rejects all missing values", {
  temp <- rep(NA, 50)
  pars <- matrix(c(0, 0, 1, 0, 0.1), nrow = 1)

  expect_error(
    Site_parCI(temp, 1, pars, 100),
    "contains only missing values"
  )
})


test_that("Site_parCI rejects matrices with more than one column", {
  temp <- matrix(rnorm(100), ncol = 2)
  pars <- matrix(c(0, 0, 1, 0, 0.1), nrow = 1)

  expect_error(
    Site_parCI(temp, 1, pars, 100),
    "single-column matrix"
  )
})


test_that("Site_parCI rejects non-numeric site_par", {
  temp <- rnorm(50)

  pars <- matrix(
    c("a", "b", "c", "d", "e"),
    nrow = 1
  )

  expect_error(
    Site_parCI(temp, 1, pars, 100),
    "`site_par` must be numeric"
  )
})


test_that("Site_parCI rejects site_par with incorrect dimensions", {
  temp <- rnorm(50)

  pars <- matrix(1:10, nrow = 2)

  expect_error(
    Site_parCI(temp, 1, pars, 100),
    "exactly 1 row and 5 columns"
  )
})


test_that("Site_parCI rejects missing values in site_par", {
  temp <- rnorm(50)

  pars <- matrix(
    c(0, 0, 1, NA, 0.1),
    nrow = 1
  )

  expect_error(
    Site_parCI(temp, 1, pars, 100),
    "cannot contain missing or infinite values"
  )
})


test_that("Site_parCI rejects non-positive scale parameters", {
  temp <- rnorm(50)

  pars <- matrix(
    c(0, 0, -1, 0, 0.1),
    nrow = 1
  )

  expect_error(
    Site_parCI(temp, 1, pars, 100),
    "scale parameter becomes non-positive"
  )
})


test_that("Site_parCI works when shape parameter equals zero", {
  set.seed(123)

  temp <- rnorm(50)

  pars <- matrix(
    c(0, 0.01, 1, 0.001, 0),
    nrow = 1
  )

  result <- Site_parCI(
    temp,
    model = 2,
    site_par = pars,
    n.boots = 100
  )

  expect_true(is.matrix(result))
  expect_identical(dim(result), c(2L, 5L))
})

#' Time-varying parameters of the best fitted GEV model
#'
#' This function fits four time-varying GEV models (with different assumptions
#' on non-stationarity in location and/or scale) to each temperature series,
#' computes the AICc of each model, and selects the best one according to the
#' lowest total AICc.
#'
#' @param add.data
#' A numeric matrix of air temperature data as calculated by `Dataset_add()`.
#'
#' @returns
#' A list with:
#' \describe{
#'   \item{best}{
#'     Index (1--4) of the model with the lowest total AICc across all sites.
#'   }
#'   \item{atsite.models}{
#'     Data frame containing the estimated parameters
#'     (`mu0`, `mu1`, `sigma0`, `sigma1`, `shape`)
#'     and sample size for each site.
#'   }
#' }
#'
#' @details
#' Model fitting is performed via `ismev::gev.fit()`.  Four nested models
#' are considered:
#' \enumerate{
#'   \item Stationary (constant location and scale).
#'   \item Time-varying location only.
#'   \item Time-varying scale only.
#'   \item Time-varying location and scale.
#' }
#' For each site the function tries up to five optimisation methods
#' (`Nelder-Mead`, `BFGS`, `CG`, `L-BFGS-B`, `SANN`)
#' and uses the first that converges.  Model selection is based on the sum of
#' site-level AICc values.
#'
#' @importFrom ismev gev.fit
#' @importFrom stats na.omit
#' @importFrom spsUtil quiet
#' @export
#'
#' @examples
#' add.data <- Dataset_add(TmaxCPC_SP)
#' best.parms <- Best_model(add.data = add.data$add_data)

Best_model <- function(add.data) {
  if (!is.matrix(add.data) && !is.data.frame(add.data)) {
    stop("Input 'add.data' must be a matrix or data frame.", call. = FALSE)
  }

  add.data <- as.matrix(add.data)

  if (ncol(add.data) < 1L) {
    stop("'add.data' must contain at least one site.", call. = FALSE)
  }

  if (min(colSums(!is.na(add.data))) < 10L) {
    stop("All sites must have at least 10 observations.", call. = FALSE)
  }

  n.sites <- ncol(add.data)
  all_pars <- lapply(1:4, function(m) matrix(NA_real_, n.sites, 5))
  aic_mat <- matrix(Inf, n.sites, 4)
  sizes <- integer(n.sites)

  for (i in seq_len(n.sites)) {
    local <- na.omit(add.data[, i])
    sizes[i] <- length(local)
    if (sizes[i] == 0L) {
      next
    }

    fit <- spsUtil::quiet(fit.models(local, seq_len(sizes[i])))

    aic_mat[i, ] <- fit$at.site.AIC
    for (m in 1:4) {
      all_pars[[m]][i, ] <- fit$pars[[m]]
    }
  }

  total_AIC <- colSums(aic_mat, na.rm = TRUE)

  if (all(is.infinite(total_AIC))) {
    stop("All model fits failed.", call. = FALSE)
  }

  best <- which.min(total_AIC)
  atsite.models <- as.data.frame(all_pars[[best]])
  atsite.models$size <- sizes
  colnames(atsite.models) <- c(
    "mu0",
    "mu1",
    "sigma0",
    "sigma1",
    "shape",
    "size"
  )

  list(best = best, atsite.models = atsite.models)
}

# =============================================================================
# Best_model.R
# Internal helpers
# =============================================================================

# -----------------------------------------------------------------------------
# Package-level constants — defined once, referenced everywhere
# -----------------------------------------------------------------------------

#' @noRd
GEV_MODEL_SPECS <- list(
  list(mul = NULL, sigl = NULL),
  list(mul = 1L, sigl = NULL),
  list(mul = NULL, sigl = 1L),
  list(mul = 1L, sigl = 1L)
)

# Positional indices into the 5-element parameter vector
# (mu0, mu1, sigma0, sigma1, shape) that each model's MLE fills.
# Slots absent from a model are fixed at zero.
#' @noRd
GEV_PAR_MAP <- list(
  c(1L, 3L, 5L),
  c(1L, 2L, 3L, 5L),
  c(1L, 3L, 4L, 5L),
  c(1L, 2L, 3L, 4L, 5L)
)

#' @noRd
GEV_K_VALS <- c(3L, 4L, 4L, 5L)

#' @noRd
OPTIM_METHODS <- c("Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN")


# -----------------------------------------------------------------------------
# Internal: fit one GEV model, cycling through optimisers until one succeeds.
# Returns the fit object or NULL on total failure.
# -----------------------------------------------------------------------------
#' @noRd
try_model <- function(local, time, model_id) {
  spec <- GEV_MODEL_SPECS[[model_id]]

  for (method in OPTIM_METHODS) {
    result <- try(
      {
        fit <- ismev::gev.fit(
          local,
          ydat = as.matrix(time),
          mul = spec$mul,
          sigl = spec$sigl,
          shl = NULL,
          mulink = identity,
          siglink = identity,
          shlink = identity,
          method = method,
          maxit = 10000L,
          show = FALSE
        )
        if (is.null(fit$mle)) {
          stop("Fit failed.", call. = FALSE)
        }
        return(fit)
      },
      silent = TRUE
    )

    if (!inherits(result, "try-error")) return(result)
  }

  NULL
}


# -----------------------------------------------------------------------------
# Internal: extract a 5-element parameter vector from a fitted model,
# placing MLE estimates in the correct slots and zeroing the rest.
# -----------------------------------------------------------------------------
#' @noRd
extract_pars <- function(model, model_id) {
  if (is.null(model)) {
    return(rep(NA_real_, 5))
  }
  out <- numeric(5)
  out[GEV_PAR_MAP[[model_id]]] <- model$mle
  out
}


# -----------------------------------------------------------------------------
# Internal: compute AICc, returning Inf when the fit is unusable.
# -----------------------------------------------------------------------------
#' @noRd
safe_AICc <- function(model, k, n) {
  nllh <- model$nllh
  if (is.null(nllh) || is.na(nllh) || n <= k + 1L) {
    return(Inf)
  }
  AIC <- 2 * k + 2 * nllh
  AIC + (2 * k * (k + 1L)) / (n - k - 1L)
}


# -----------------------------------------------------------------------------
# Internal: fit all four GEV models to one site and return parameters + AICc.
# -----------------------------------------------------------------------------
#' @noRd
fit.models <- function(local, time) {
  n <- length(local)
  models <- lapply(1:4, try_model, local = local, time = time)

  list(
    pars = lapply(1:4, function(i) extract_pars(models[[i]], i)),
    at.site.AIC = mapply(safe_AICc, models, GEV_K_VALS, MoreArgs = list(n = n))
  )
}

test_that("Best_model returns correct structure", {
  set.seed(123)
  fake_data <- matrix(rnorm(90), ncol = 3) # 30 years × 3 sites

  result <- Best_model(fake_data)

  # Output is a list
  expect_type(result, "list")
  expect_named(result, c("best", "atsite.models"))

  # Best model index
  expect_true(result$best %in% 1:4)

  # atsite.models structure
  expect_s3_class(result$atsite.models, "data.frame")

  expect_named(
    result$atsite.models,
    c("mu0", "mu1", "sigma0", "sigma1", "shape", "size")
  )

  expect_identical(ncol(result$atsite.models), 6L)

  # Number of rows = number of sites
  expect_identical(
    nrow(result$atsite.models),
    ncol(fake_data)
  )
})


test_that("Best_model stores correct sample sizes", {
  set.seed(123)

  fake_data <- matrix(rnorm(90), ncol = 3)

  result <- Best_model(fake_data)

  expect_true(
    all(result$atsite.models$size == 30)
  )
})


test_that("Best_model throws error with invalid input type", {
  expect_error(
    Best_model("not a matrix"),
    "must be a matrix or data frame"
  )
})


test_that("Best_model throws error when there are no sites", {
  empty_data <- matrix(numeric(0), nrow = 10, ncol = 0)

  expect_error(
    Best_model(empty_data),
    "must contain at least one site"
  )
})


test_that("Best_model throws error when any site has fewer than 10 observations", {
  set.seed(123)

  fake_data <- matrix(rnorm(90), ncol = 3)

  # Site 1 will have only 5 observations
  fake_data[1:25, 1] <- NA

  expect_error(
    Best_model(fake_data),
    "All sites must have at least 10 observations"
  )
})


test_that("Best_model returns finite sample sizes", {
  set.seed(123)

  fake_data <- matrix(rnorm(90), ncol = 3)

  result <- Best_model(fake_data)

  expect_true(
    all(is.finite(result$atsite.models$size))
  )
})


test_that("Best_model returns one row per site", {
  set.seed(123)

  fake_data <- matrix(rnorm(150), ncol = 5)

  result <- Best_model(fake_data)

  expect_identical(
    nrow(result$atsite.models),
    5L
  )
})


test_that("Best_model parameter columns are numeric", {
  set.seed(123)

  fake_data <- matrix(rnorm(90), ncol = 3)

  result <- Best_model(fake_data)

  expect_true(
    all(
      sapply(result$atsite.models, is.numeric)
    )
  )
})


test_that("Best_model returns finite parameter estimates", {
  set.seed(123)

  fake_data <- matrix(rnorm(90), ncol = 3)

  result <- Best_model(fake_data)

  expect_true(
    all(
      is.finite(
        as.matrix(result$atsite.models)
      )
    )
  )
})


test_that("Best_model returns a valid best model index", {
  set.seed(123)

  fake_data <- matrix(rnorm(120), ncol = 4)

  result <- Best_model(fake_data)

  expect_length(
    result$best,
    1
  )

  expect_true(
    result$best %in% 1:4
  )
})

test_that("Best_model accepts a data.frame input", {
  set.seed(123)

  fake_data <- as.data.frame(
    matrix(rnorm(90), ncol = 3)
  )

  result <- Best_model(fake_data)

  expect_type(result, "list")

  expect_identical(
    nrow(result$atsite.models),
    3L
  )
})

test_that("Best_model stores sample sizes correctly with missing values", {
  set.seed(123)

  fake_data <- matrix(rnorm(90), ncol = 3)

  fake_data[1:5, 1] <- NA
  fake_data[1:10, 2] <- NA

  result <- Best_model(fake_data)

  expect_identical(result$atsite.models$size[1], 25L)
  expect_identical(result$atsite.models$size[2], 20L)
  expect_identical(result$atsite.models$size[3], 30L)
})

test_that("Best_model works with a single site", {
  set.seed(123)

  fake_data <- matrix(rnorm(50), ncol = 1)

  result <- Best_model(fake_data)

  expect_identical(
    nrow(result$atsite.models),
    1L
  )

  expect_true(
    result$best %in% 1:4
  )
})

test_that("Best_model returns an integer model index", {
  set.seed(123)

  fake_data <- matrix(rnorm(120), ncol = 4)

  result <- Best_model(fake_data)

  expect_type(
    result$best,
    "integer"
  )

  expect_length(
    result$best,
    1
  )

  expect_true(
    result$best %in% 1:4
  )
})

#' Extreme air temperature data
#'
#' Daily extremes for air temperature data sampled from
#' the block maxima approach. A block corresponds to one year.
#' The data belongs to The Climate Prediction Center - NOAA.
#' location of Ribeirao Preto, State of Sao Paulo in Brazil.
#'
#' @format ## `TmaxCPC_SP`
#' A data.frame with 11 columns and 34 rows.
#'
#' \describe{
#' \item{Year}{Year}
#' \item{Pixel_1 to Pixel_10}{daily extremes for air temperature}
#' }
#' @source <https://psl.noaa.gov/>
"TmaxCPC_SP"

#' Add_RegProb
#'
#' @param quantiles
#' A numeric vector with no missing data containing extreme air temperatures.
#' @param regional_pars
#' A 5-column, 1-row `data.frame` or `matrix` as generated by `Reg_par()`.
#' @param site_temp
#' A non-empty numeric vector or 1-column matrix of site air temperature data.
#' @param n.year
#' A single integer giving the year index for which parameters are calculated.
#' Must be between 1 and `length(site_temp)`.
#'
#' @returns
#' A numeric matrix of cumulative probabilities (clamped to \[0.0001, 0.9999\])
#' associated with the input quantiles from the regional GEV distribution.
#'
#' @export
#' @importFrom extRemes pevd
#'
#'
#' @examples
#' quantiles <- c(37.8, 37.9, 38.0, 38.1, 38.2, 38.5, 39.0)
#' add.data <- Dataset_add(TmaxCPC_SP)
#' best.parms <- Best_model(add.data = add.data$add_data)
#' regional_pars <- Reg_par(best_model = best.parms$atsite.models)
#' Add_RegProb(
#'   quantiles     = quantiles,
#'   regional_pars = regional_pars,
#'   site_temp     = TmaxCPC_SP$Pixel_1,
#'   n.year        = 34
#' )
Add_RegProb <- function(quantiles, regional_pars, site_temp, n.year) {
  check_quantiles(quantiles) # defined in input_checks.R
  regional_pars <- check_reg_par(regional_pars) # reuses existing helper
  check_site_temp(site_temp) # defined in input_checks.R
  check_n.year(n.year, site_temp) # defined in input_checks.R

  site_mean <- mean(site_temp, na.rm = TRUE)
  loc <- regional_pars[1] + regional_pars[2] * n.year
  scale <- regional_pars[3] + regional_pars[4] * n.year
  shape <- regional_pars[5]

  if (scale <= 0) {
    stop("Calculated scale parameter must be positive.", call. = FALSE)
  }

  RegProb <- extRemes::pevd(
    quantiles - site_mean,
    loc = loc,
    scale = scale,
    shape = shape,
    type = "GEV"
  )

  # Clamp to valid probability range
  RegProb <- pmax(pmin(RegProb, 0.9999), 0.0001)

  as.matrix(RegProb)
}

test_that("Add_RegProb returns probabilities between 0.0001 and 0.9999", {
  quantiles <- c(39.5, 40.0, 40.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05, 0.1), nrow = 1))
  site_temp <- rnorm(100, mean = 40, sd = 2)
  n.year <- 50

  result <- Add_RegProb(quantiles, regional_pars, site_temp, n.year)

  expect_true(all(result >= 0.0001))
  expect_true(all(result <= 0.9999))
  expect_type(result, "double")
  expect_true(all(is.finite(result)))
})

test_that("Add_RegProb throws error with missing quantiles", {
  quantiles <- c(39.5, NA, 40.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05, 0.1), nrow = 1))
  site_temp <- rnorm(100, mean = 40, sd = 2)
  n.year <- 50

  expect_error(
    Add_RegProb(quantiles, regional_pars, site_temp, n.year),
    regexp = "must be a numeric vector with no missing values"
  )
})

test_that("Add_RegProb throws error for invalid regional_pars dimensions", {
  quantiles <- c(39.5, 40.0, 40.5)
  wrong_regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05), nrow = 1)) # Only 4 columns
  site_temp <- rnorm(100, mean = 40, sd = 2)
  n.year <- 30

  expect_error(
    Add_RegProb(quantiles, wrong_regional_pars, site_temp, n.year),
    "`reg_par` must have exactly 1 row and 5 columns.",
    fixed = TRUE
  )
})

test_that("Add_RegProb throws error for non-numeric site_temp", {
  quantiles <- c(39.5, 40.0, 40.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05, 0.1), nrow = 1))
  site_temp <- as.character(rnorm(100, mean = 40, sd = 2)) # Non-numeric
  n.year <- 30

  expect_error(
    Add_RegProb(quantiles, regional_pars, site_temp, n.year),
    regexp = "must be a non-empty numeric vector"
  )
})


test_that("Add_RegProb throws error if scale parameter becomes non-positive", {
  quantiles <- c(39.5, 40.0, 40.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, -2, -0.05, 0.1), nrow = 1)) # Negative scale intercept and slope
  site_temp <- rnorm(100, mean = 40, sd = 2)
  n.year <- 30

  expect_error(
    Add_RegProb(quantiles, regional_pars, site_temp, n.year),
    regexp = "scale parameter must be positive"
  )
})

test_that("Add_RegProb returns correct number of probabilities", {
  quantiles <- seq(38, 42, by = 0.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05, 0.1), nrow = 1))
  site_temp <- rnorm(100, mean = 40, sd = 2)
  n.year <- 20

  result <- Add_RegProb(quantiles, regional_pars, site_temp, n.year)
  expect_length(result, length(quantiles))
})

test_that("Add_RegProb throws error for non-integer n.year", {
  quantiles <- c(39.5, 40.0, 40.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05, 0.1), nrow = 1))
  site_temp <- rnorm(100, mean = 40, sd = 2)

  expect_error(
    Add_RegProb(
      quantiles,
      regional_pars,
      site_temp,
      n.year = 20.5
    ),
    regexp = "`n.year`"
  )
})

test_that("Add_RegProb throws error for empty site_temp", {
  quantiles <- c(39.5, 40.0, 40.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05, 0.1), nrow = 1))

  expect_error(
    Add_RegProb(
      quantiles,
      regional_pars,
      numeric(0),
      n.year = 1
    ),
    regexp = "must be a non-empty numeric vector"
  )
})

test_that("Add_RegProb is deterministic", {
  quantiles <- seq(38, 42, by = 0.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05, 0.1), nrow = 1))
  site_temp <- rnorm(100, mean = 40, sd = 2)
  n.year <- 20

  result1 <- Add_RegProb(
    quantiles,
    regional_pars,
    site_temp,
    n.year
  )

  result2 <- Add_RegProb(
    quantiles,
    regional_pars,
    site_temp,
    n.year
  )

  expect_identical(result1, result2)
})

test_that("Add_RegProb works for the first year", {
  quantiles <- seq(38, 42, by = 0.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05, 0.1), nrow = 1))
  site_temp <- rnorm(100, mean = 40, sd = 2)

  result <- Add_RegProb(
    quantiles,
    regional_pars,
    site_temp,
    n.year = 1
  )

  expect_true(all(is.finite(result)))
})

test_that("Add_RegProb works for the last year", {
  quantiles <- seq(38, 42, by = 0.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05, 0.1), nrow = 1))
  site_temp <- rnorm(100, mean = 40, sd = 2)

  result <- Add_RegProb(
    quantiles,
    regional_pars,
    site_temp,
    n.year = length(site_temp)
  )

  expect_true(all(is.finite(result)))
})

test_that("Add_RegProb returns increasing probabilities for increasing quantiles", {
  quantiles <- seq(38, 42, by = 0.5)
  regional_pars <- data.frame(matrix(c(40, 0.1, 2, 0.05, 0.1), nrow = 1))
  site_temp <- rnorm(100, mean = 40, sd = 2)

  result <- Add_RegProb(
    quantiles,
    regional_pars,
    site_temp,
    n.year = 20
  )

  expect_true(all(diff(as.vector(result)) >= 0))
})

test_that("Add_RegQuant works with valid inputs", {
  prob <- c(0.8, 0.9, 0.95)
  regional_pars <- matrix(c(10, 0.5, 5, 0.2, 0.1), nrow = 1)
  site_temp <- seq(25, 35, length.out = 50)
  n.year <- 10

  result <- Add_RegQuant(prob, regional_pars, site_temp, n.year)

  expect_type(result, "double")
  expect_length(result, length(prob))
  expect_named(result)
  expect_false(anyNA(result))
  expect_true(all(is.finite(result)))
})

test_that("Add_RegQuant errors with invalid prob", {
  rp <- matrix(c(10, 0.5, 5, 0.2, 0.1), nrow = 1)
  temp <- rep(30, 50)

  expect_error(
    Add_RegQuant(c(1.2, 0.9), rp, temp, 5),
    "must be a numeric vector with values strictly between 0 and 1"
  )
  expect_error(
    Add_RegQuant(c(0.8, NA), rp, temp, 5),
    "must be a numeric vector with values strictly between 0 and 1"
  )
})

test_that("Add_RegQuant errors with invalid regional_pars", {
  temp <- rep(30, 50)
  expect_error(
    Add_RegQuant(0.9, matrix(1:4, nrow = 1), temp, 5),
    "`reg_par` must have exactly 1 row and 5 columns.",
    fixed = TRUE
  )
  expect_error(
    Add_RegQuant(0.9, matrix(1:10, nrow = 2), temp, 5),
    "`reg_par` must have exactly 1 row and 5 columns.",
    fixed = TRUE
  )
})

test_that("Add_RegQuant errors with invalid site_temp", {
  rp <- matrix(c(10, 0.5, 5, 0.2, 0.1), nrow = 1)
  expect_error(
    Add_RegQuant(0.9, rp, NA, 5),
    "`site_temp` must be a non-empty numeric vector or 1-column matrix.",
    fixed = TRUE
  )
  expect_error(
    Add_RegQuant(0.9, rp, numeric(0), 5),
    "`site_temp` must be a non-empty numeric vector or 1-column matrix.",
    fixed = TRUE
  )
})

test_that("Add_RegQuant errors with invalid n.year", {
  rp <- matrix(c(10, 0.5, 5, 0.2, 0.1), nrow = 1)
  temp <- rep(30, 50)

  expect_error(
    Add_RegQuant(0.9, rp, temp, 0),
    "`n.year` must be between 1 and length(`site_temp`).",
    fixed = TRUE
  )
  expect_error(
    Add_RegQuant(0.9, rp, temp, 100),
    "`n.year` must be between 1 and length(`site_temp`)",
    fixed = TRUE
  )
  expect_error(
    Add_RegQuant(0.9, rp, temp, "five"),
    "`n.year` must be a single numeric value.",
    fixed = TRUE
  )
})

test_that("Add_RegQuant errors with non-integer n.year", {
  rp <- matrix(c(10, 0.5, 5, 0.2, 0.1), nrow = 1)
  temp <- rep(30, 50)

  expect_error(
    Add_RegQuant(0.9, rp, temp, 10.5),
    "`n.year` must be a whole number.",
    fixed = TRUE
  )
})

test_that("Add_RegQuant errors when scale parameter becomes non-positive", {
  prob <- 0.9
  site_temp <- rep(30, 50)

  regional_pars <- matrix(
    c(10, 0.5, -2, -0.2, 0.1),
    nrow = 1
  )

  expect_error(
    Add_RegQuant(
      prob,
      regional_pars,
      site_temp,
      n.year = 20
    ),
    "Calculated scale parameter must be positive.",
    fixed = TRUE
  )
})

test_that("Add_RegQuant and Add_RegProb are approximately inverse functions", {
  prob <- c(0.90, 0.95, 0.99)

  regional_pars <- matrix(
    c(10, 0.5, 5, 0.2, 0.1),
    nrow = 1
  )

  site_temp <- rep(30, 50)

  quantiles <- Add_RegQuant(
    prob,
    regional_pars,
    site_temp,
    n.year = 10
  )

  recovered_prob <- Add_RegProb(
    quantiles,
    regional_pars,
    site_temp,
    n.year = 10
  )

  expect_equal(
    as.vector(recovered_prob),
    prob,
    tolerance = 1e-6
  )
})

test_that("Add_RegQuant returns correctly named quantiles", {
  prob <- c(0.90, 0.95, 0.99)
  regional_pars <- matrix(c(10, 0.5, 5, 0.2, 0.1), nrow = 1)
  site_temp <- rep(30, 50)

  result <- Add_RegQuant(
    prob,
    regional_pars,
    site_temp,
    n.year = 10
  )

  expect_named(
    result,
    c("Q90", "Q95", "Q99")
  )
})

test_that("Add_RegQuant returns increasing quantiles for increasing probabilities", {
  prob <- c(0.80, 0.90, 0.95, 0.99)
  regional_pars <- matrix(c(10, 0.5, 5, 0.2, 0.1), nrow = 1)
  site_temp <- rep(30, 50)

  result <- Add_RegQuant(
    prob,
    regional_pars,
    site_temp,
    n.year = 10
  )

  expect_true(all(diff(result) > 0))
})

test_that("Add_RegQuant works for the first year", {
  prob <- 0.9
  regional_pars <- matrix(c(10, 0.5, 5, 0.2, 0.1), nrow = 1)
  site_temp <- rep(30, 50)

  result <- Add_RegQuant(
    prob,
    regional_pars,
    site_temp,
    n.year = 1
  )

  expect_true(all(is.finite(result)))
})

test_that("Add_RegQuant works for the last year", {
  prob <- 0.9
  regional_pars <- matrix(c(10, 0.5, 5, 0.2, 0.1), nrow = 1)
  site_temp <- rep(30, 50)

  result <- Add_RegQuant(
    prob,
    regional_pars,
    site_temp,
    n.year = length(site_temp)
  )

  expect_true(all(is.finite(result)))
})

#' Regional Quantile Estimation for a Site Using Time-Varying GEV Parameters
#'
#' Computes quantiles for a given site based on regional GEV parameters and
#' a time-varying location and scale model. Estimated quantiles are adjusted
#' by the site's mean temperature.
#'
#' @param prob
#' A numeric vector of probabilities strictly between 0 and 1, with no
#' missing values.
#' @param regional_pars
#' A 1-row, 5-column `matrix` or `data.frame` containing regional
#' GEV parameters: location intercept, location slope, scale intercept,
#' scale slope, and shape.
#' @param site_temp
#' A numeric vector or 1-column matrix of site air temperature data.
#' @param n.year
#' A single integer indicating the year index for which quantiles are
#' calculated. Must be between 1 and `length(site_temp)`.
#'
#' @returns
#' A named numeric vector of adjusted regional quantiles for the specified
#' year. Names are of the form `"Q90"`, `"Q95"``, etc.
#'
#' @export
#' @importFrom extRemes qevd
#'
#' @examples
#' prob <- c(0.90, 0.92, 0.93, 0.94, 0.95, 0.97, 0.99)
#' add.data <- Dataset_add(TmaxCPC_SP)
#' best.parms <- Best_model(add.data = add.data$add_data)
#' regional_pars <- Reg_par(best_model = best.parms$atsite.models)
#' Add_RegQuant(
#'   prob          = prob,
#'   regional_pars = regional_pars,
#'   site_temp     = TmaxCPC_SP$Pixel_1,
#'   n.year        = 34
#' )
Add_RegQuant <- function(prob, regional_pars, site_temp, n.year) {
  check_prob(prob) # defined in input_checks.R
  regional_pars <- check_reg_par(regional_pars) # reuses existing helper
  check_site_temp(site_temp) # defined in input_checks.R
  check_n.year(n.year, site_temp) # defined in input_checks.R

  loc <- regional_pars[1] + regional_pars[2] * n.year
  scale_val <- regional_pars[3] + regional_pars[4] * n.year
  shape <- regional_pars[5]

  if (scale_val <= 0) {
    stop("Calculated scale parameter must be positive.", call. = FALSE)
  }

  Qt <- extRemes::qevd(
    prob,
    loc = loc,
    scale = scale_val,
    shape = shape,
    type = "GEV"
  ) +
    mean(site_temp, na.rm = TRUE)

  names(Qt) <- paste0("Q", prob * 100)
  Qt
}

#' Discordance measures calculated from the additive approach
#'
#' This function calculates the Hosking and Wallis' discordance measure (discord) and
#' the robust discordance measure (Neykov et al. 2007) <doi:10.1029/2006WR005322> using
#' the additive approach proposed in Martins et al. (2022) <doi:10.1590/1678-4499.20220061>.
#'
#' @param dataset A numeric matrix with extreme air temperature data from multiple sites.
#'   The first column must contain the years, and the remaining columns contain
#'   temperature data from each site.
#'
#' @returns A `data.frame` with 8 columns:
#' \describe{
#'     \item{Local}{Site identifier.}
#'     \item{SampleSize}{Number of observations for each site.}
#'     \item{l_1}{First L-moment (mean).}
#'     \item{l_2}{Second L-moment (L-scale).}
#'     \item{t_3}{L-skewness ratio.}
#'     \item{t_4}{L-kurtosis ratio.}
#'     \item{t_5}{Fifth-order L-moment ratio.}
#'     \item{discord}{Original discordance statistic indicating potential outlier status.}
#' }
#'
#' @details The discordance measures identify sites that are potentially discordant
#'   within a regional frequency analysis framework for extreme air temperature series.
#'
#' @export
#' @importFrom lmomRFA regsamlmu
#' @importFrom rrcov getDistance Cov
#'
#' @examples
#' Add_Discord(TmaxCPC_SP)

Add_Discord <- function(dataset) {
  if (ncol(dataset) < 2) {
    stop(
      "The dataset must contain a year column and at least one site.",
      call. = FALSE
    )
  }

  if (anyNA(dataset[, 1])) {
    stop("Column 'Years' cannot have missing data.", call. = FALSE)
  }

  n_total <- ncol(dataset)

  dataset.year <- dataset[, 2:n_total, drop = FALSE]

  n_sites <- ncol(dataset.year)

  if (n_sites < 3) {
    stop("The number of sites should be at least 3.", call. = FALSE)
  }

  min_sample_size <- min(colSums(!is.na(dataset.year)))

  if (min_sample_size < 10) {
    stop("All sites must have at least 10 years of records.", call. = FALSE)
  }

  d <- as.data.frame(matrix(NA_real_, n_sites, 8))

  d[, 1:7] <- regsamlmu(dataset.year, lcv = FALSE)
  d[, 8] <- sqrt(getDistance(Cov(d[, 4:6])))

  colnames(d) <- c(
    "Local",
    "SampleSize",
    "l_1",
    "l_2",
    "t_3",
    "t_4",
    "t_5",
    "discord"
  )

  return(d)
}

test_that("Add_Discord returns expected structure", {
  set.seed(123)
  years <- 1980:2020
  temp_data <- matrix(rnorm(length(years) * 8, mean = 30, sd = 5), ncol = 8)
  dataset <- cbind(Years = years, temp_data)

  result <- Add_Discord(dataset)

  expect_s3_class(result, "data.frame")
  expect_identical(ncol(result), 8L)
  expect_named(
    result,
    c("Local", "SampleSize", "l_1", "l_2", "t_3", "t_4", "t_5", "discord")
  )
  expect_identical(nrow(result), 8L)
})

test_that("Add_Discord throws error when Years column has NAs", {
  set.seed(123)
  years <- 1980:2020
  years[5] <- NA
  temp_data <- matrix(rnorm(length(years) * 8, mean = 30, sd = 5), ncol = 8)
  dataset <- cbind(Years = years, temp_data)

  expect_error(Add_Discord(dataset), "Column 'Years' cannot have missing data.")
})

test_that("Add_Discord throws error when less than 3 sites", {
  years <- 1980:2020
  temp_data <- matrix(rnorm(length(years) * 6, mean = 30, sd = 5), ncol = 2)
  dataset <- cbind(Years = years, temp_data)

  expect_error(
    Add_Discord(dataset),
    "The number of sites should be at least 3."
  )
})

test_that("Add_Discord throws error when any site has fewer than 10 observations", {
  years <- 1980:2020
  temp_data <- matrix(rnorm(length(years) * 8, mean = 30, sd = 5), ncol = 8)
  temp_data[1:32, 3] <- NA # 41 - 32 = 9 valid values in site 3
  dataset <- cbind(Years = years, temp_data)

  expect_error(
    Add_Discord(dataset),
    "All sites must have at least 10 years of records."
  )
})

test_that("Discord measures are non-negative", {
  years <- 1980:2020
  temp_data <- matrix(rnorm(length(years) * 8, mean = 30, sd = 5), ncol = 8)
  dataset <- cbind(Years = years, temp_data)

  result <- Add_Discord(dataset)

  expect_true(all(is.finite(result$discord)))
  expect_true(all(result$discord >= 0))
})

test_that("Add_Discord returns an error when dataset has fewer than two columns", {
  years <- 1980:2020
  dataset <- matrix(years, ncol = 1)

  expect_error(
    Add_Discord(dataset),
    "The dataset must contain a year column and at least one site."
  )
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
set.seed(23)
options(NSTempRFA.progress = FALSE) # suppress progress bar for session

## ----setup--------------------------------------------------------------------
library(NSTempRFA)

## ----Table 1------------------------------------------------------------------
Table1 <- lonlat_Tmax
Table1

## ----Table 2------------------------------------------------------------------
TmaxCPC_SP[, 2:11] <- round(TmaxCPC_SP[, 2:11], 1)
Table2 <- TmaxCPC_SP
Table2

## ----Add_Discord--------------------------------------------------------------
Add_Discord(TmaxCPC_SP)

## ----Preparing the data-------------------------------------------------------
add.data <- Dataset_add(TmaxCPC_SP)
add.data$add_data
add.data$reg_mean

## ----Add_H--------------------------------------------------------------------
set.seed(123)
rho <- 0.49 # The average spatial correlation among the series
Ns <- 500 # Number of Simulation required for calculating
Add_H <- Add_Heterogeneity(dataset.add = add.data$add_data, rho = rho, Ns = Ns)
Add_H

## ----Add_d--------------------------------------------------------------------
Add_Discord(TmaxCPC_SP)

## ----Best_model---------------------------------------------------------------
best.parms <- Best_model(add.data = add.data$add_data)
# The best model is:
as.numeric(best.parms$best) #Model 2
# The at-site parameters are:
best.parms$atsite.models

## ----Reg_par------------------------------------------------------------------
regional.parms <- Reg_par(best_model = best.parms$atsite.models)
regional.parms

## ----Reg_parCI----------------------------------------------------------------
set.seed(123)
Reg_parCI(
  add_data = add.data$add_data,
  model = best.parms$best,
  reg_par = regional.parms,
  n.boots = 500
)

## ----Site_parCI---------------------------------------------------------------
set.seed(123)
temperatures <- TmaxCPC_SP$Pixel_1
model <- 2
site_par <- Fit_model(temperatures, model)
if (all(is.finite(as.numeric(site_par[1, 1:5])))) {
  Site_parCI(
    atsite_temp = temperatures,
    model = model,
    site_par = site_par[1, 1:5],
    n.boots = 500
  )
}

## ----Add_RegQuant-------------------------------------------------------------
prob <- c(0.8, 0.85, 0.90, 0.92, 0.93, 0.94, 0.95, 0.97, 0.99)
add.data <- Dataset_add(TmaxCPC_SP)
best.parms <- Best_model(add.data = add.data$add_data)
regional_pars <- Reg_par(best_model = best.parms$atsite.models)
Quantiles_1991 <- Add_RegQuant(
  prob = prob,
  regional_pars = regional_pars,
  site_temp = TmaxCPC_SP$Pixel_1,
  n.year = 1
)
Quantiles_2024 <- Add_RegQuant(
  prob = prob,
  regional_pars = regional_pars,
  site_temp = TmaxCPC_SP$Pixel_1,
  n.year = 34
)
Quantiles_1991
Quantiles_2024

## ----Add_RegProb--------------------------------------------------------------
 quantiles <- c(
   35.58301,
   35.81863,
   36.11727,
   36.26792,
   36.35403,
   36.44996,
   36.55892,
   36.84087,
   37.35040
 )
 add.data <- Dataset_add(TmaxCPC_SP)
 best.parms <- Best_model(add.data = add.data$add_data)
 regional_pars <- Reg_par(best_model = best.parms$atsite.models)
 Probs_1991 <- Add_RegProb(
   quantiles = quantiles,
   regional_pars = regional_pars,
   site_temp = TmaxCPC_SP$Pixel_1,
   n.year = 1
 )
 Probs_2024 <- Add_RegProb(
   quantiles = quantiles,
   regional_pars = regional_pars,
   site_temp = TmaxCPC_SP$Pixel_1,
   n.year = 34
 )
 Probs_1991
 Probs_2024

#' Estimate the parameters of the regional distribution
#'
#' @param best_model
#' A 6-column `data.frame` as generated by `Best_model()`.
#' \itemize{
#'   \item Columns 1--5: `mu0`, `mu1`, `sigma0`,
#'         `sigma1`, `shape`.
#'   \item Column 6: `size` (sample size per site, used as weight).
#' }
#'
#' @returns
#' A one-row `data.frame` with the size-weighted regional parameters of a
#' time-varying generalized extreme value distribution.
#'
#' @export
#' @importFrom stats weighted.mean
#'
#' @examples
#' add.data <- Dataset_add(TmaxCPC_SP)
#' best.parms <- Best_model(add.data = add.data$add_data)
#' Reg_par(best_model = best.parms$atsite.models)

Reg_par <- function(best_model) {
  check_best_model_df(best_model)

  w <- best_model$size
  par_cols <- c("mu0", "mu1", "sigma0", "sigma1", "shape")

  # Replace non-finite parameter values with NA so they are excluded
  # from weighted.mean() along with genuinely missing fits.
  pars <- best_model[par_cols]
  pars[!is.finite(as.matrix(pars))] <- NA_real_

  wm <- vapply(
    pars,
    weighted.mean,
    w = w,
    na.rm = TRUE,
    FUN.VALUE = numeric(1)
  )

  names(wm) <- paste0("weighted_", par_cols)
  as.data.frame(as.list(wm))
}

test_that("Reg_par returns a one-row data frame with five columns", {
  best_model <- data.frame(
    mu0 = c(10, 20),
    mu1 = c(1, 2),
    sigma0 = c(3, 4),
    sigma1 = c(0.1, 0.2),
    shape = c(0.05, 0.10),
    size = c(50, 100)
  )

  result <- Reg_par(best_model)

  expect_s3_class(result, "data.frame")
  expect_identical(nrow(result), 1L)
  expect_identical(ncol(result), 5L)

  expect_named(
    result,
    c(
      "weighted_mu0",
      "weighted_mu1",
      "weighted_sigma0",
      "weighted_sigma1",
      "weighted_shape"
    )
  )
})

test_that("Reg_par computes weighted means correctly", {
  best_model <- data.frame(
    mu0 = c(10, 20),
    mu1 = c(1, 3),
    sigma0 = c(2, 6),
    sigma1 = c(0.1, 0.5),
    shape = c(0.05, 0.15),
    size = c(1, 3)
  )

  result <- Reg_par(best_model)

  expect_identical(result$weighted_mu0, (10 * 1 + 20 * 3) / 4)
  expect_identical(result$weighted_mu1, (1 * 1 + 3 * 3) / 4)
  expect_identical(result$weighted_sigma0, (2 * 1 + 6 * 3) / 4)
  expect_identical(result$weighted_sigma1, (0.1 * 1 + 0.5 * 3) / 4)
  expect_identical(result$weighted_shape, (0.05 * 1 + 0.15 * 3) / 4)
})

test_that("Reg_par rejects non-data-frame input", {
  expect_error(
    Reg_par(matrix(1:12, ncol = 6)),
    "Input 'best_model' must be a data frame."
  )
})

test_that("Reg_par rejects missing columns", {
  best_model <- data.frame(
    mu0 = 1,
    mu1 = 1,
    sigma0 = 1,
    sigma1 = 1,
    size = 10
  )

  expect_error(
    Reg_par(best_model),
    "must contain the columns"
  )
})

test_that("Reg_par rejects non-numeric columns", {
  best_model <- data.frame(
    mu0 = 1,
    mu1 = 1,
    sigma0 = 1,
    sigma1 = 1,
    shape = "a",
    size = 10
  )

  expect_error(
    Reg_par(best_model),
    "All columns in 'best_model' must be numeric"
  )
})

test_that("Reg_par rejects NA values in size column", {
  best_model <- data.frame(
    mu0 = 1,
    mu1 = 1,
    sigma0 = 1,
    sigma1 = 1,
    shape = 0.1,
    size = NA_real_
  )
  expect_error(
    Reg_par(best_model),
    "The 'size' column must contain finite non-negative values.",
    fixed = TRUE
  )
})

test_that("Reg_par rejects zero total sample size", {
  best_model <- data.frame(
    mu0 = c(1, 2),
    mu1 = c(1, 2),
    sigma0 = c(1, 2),
    sigma1 = c(1, 2),
    shape = c(0.1, 0.2),
    size = c(0, 0)
  )

  expect_error(
    Reg_par(best_model),
    "The sum of the 'size' column must be greater than zero."
  )
})

test_that("Reg_par works for a single site", {
  best_model <- data.frame(
    mu0 = 10,
    mu1 = 1,
    sigma0 = 2,
    sigma1 = 0.1,
    shape = 0.05,
    size = 30
  )

  result <- Reg_par(best_model)

  expect_identical(result$weighted_mu0, 10)
  expect_identical(result$weighted_mu1, 1)
  expect_identical(result$weighted_sigma0, 2)
  expect_identical(result$weighted_sigma1, 0.1)
  expect_identical(result$weighted_shape, 0.05)
})

test_that("Reg_par handles NA values in parameter columns gracefully", {
  best_model <- data.frame(
    mu0 = c(10, NA),
    mu1 = c(1, 2),
    sigma0 = c(3, 4),
    sigma1 = c(0.1, 0.2),
    shape = c(0.05, 0.10),
    size = c(50, 100)
  )
  result <- Reg_par(best_model)
  expect_s3_class(result, "data.frame")
  # NA site is dropped by na.rm = TRUE; result is from the non-NA site only
  expect_identical(result$weighted_mu0, 10)
})

test_that("Reg_par handles Inf values in parameter columns gracefully", {
  best_model <- data.frame(
    mu0 = c(10, Inf),
    mu1 = c(1, 2),
    sigma0 = c(3, 4),
    sigma1 = c(0.1, 0.2),
    shape = c(0.05, 0.10),
    size = c(50, 100)
  )
  result <- Reg_par(best_model)
  expect_s3_class(result, "data.frame")
  expect_true(is.finite(result$weighted_mu0))
  expect_identical(result$weighted_mu0, 10) # Inf site excluded
})

test_that("Reg_par handles -Inf values in parameter columns gracefully", {
  best_model <- data.frame(
    mu0 = c(10, -Inf),
    mu1 = c(1, 2),
    sigma0 = c(3, 4),
    sigma1 = c(0.1, 0.2),
    shape = c(0.05, 0.10),
    size = c(50, 100)
  )
  result <- Reg_par(best_model)
  expect_s3_class(result, "data.frame")
  expect_true(is.finite(result$weighted_mu0))
  expect_identical(result$weighted_mu0, 10) # -Inf site excluded
})

#' Confidence intervals for regional GEV parameters
#'
#' @name Reg_parCI
#' @param add_data
#' A numeric matrix of air temperature data as calculated by `Dataset_add()`.
#' @param model
#' A single integer number from 1 to 4 defining the GEV model.
#' May be provided by `Best_model()`.
#' @param reg_par
#' A 5-column and 1-row data.frame or matrix as generated by `Reg_par()`.
#' * 1st column is the mu0 parameter,
#' * 2nd is the mu1 parameter,
#' * 3rd is the sigma0 parameter,
#' * 4th is the sigma1 parameter,
#' * 5th is the shape parameter.
#' @param n.boots
#' A single number describing the number of bootstrap replicates.
#' Whenever possible, `n.boots` should be set to 999 (default),
#' as suggested by Burn (2003) <10.1623/hysj.48.1.25.43485>
#' and O'Brien and Burn (2014) <10.1016/j.jhydrol.2014.09.041>.
#' @param progress
#' Logical scalar controlling whether a progress bar is displayed.
#' `TRUE` forces the bar on, `FALSE` forces it off, and
#' `NULL` (default) defers to `getOption("NSTempRFA.progress")`,
#' which itself falls back to `interactive()`. Set
#' `options(NSTempRFA.progress = FALSE)` to suppress the bar globally.
#'
#' @returns
#' A matrix containing the 95% confidence intervals (lower and upper bounds)
#' of the time-varying parameter estimates. The spatial dependence between
#' sites is preserved.
#'
#' @export
#' @importFrom stats quantile
#' @importFrom utils txtProgressBar setTxtProgressBar
#'
#' @examplesIf interactive()
#' add.data <- Dataset_add(TmaxCPC_SP)
#' best.parms <- Best_model(add.data = add.data$add_data)
#' regional.parms <- Reg_par(best_model = best.parms$atsite.models)
#'
#' Reg_parCI(
#'   add_data  = add.data$add_data,
#'   model     = best.parms$best,
#'   reg_par   = regional.parms,
#'   n.boots   = 100
#' )
Reg_parCI <- function(
    add_data,
    model,
    reg_par,
    n.boots = 999,
    progress = NULL
) {

  # ============================================================
  # Input checks
  # ============================================================

  check_model(model)
  check_n.boots(n.boots)

  add_data <- check_add_data(add_data)
  reg_par <- check_reg_par(reg_par)

  max_time <- nrow(add_data)

  # ============================================================
  # Construct temporal parameters
  # ============================================================

  time <- seq_len(max_time)

  par.temporal <- matrix(NA_real_, max_time, 3)

  par.temporal[, 1] <- reg_par[1] + reg_par[2] * time
  par.temporal[, 2] <- reg_par[3] + reg_par[4] * time
  par.temporal[, 3] <- reg_par[5]

  if (any(par.temporal[, 2] <= 0)) {
    stop(
      "Time-varying scale parameter became non-positive.",
      call. = FALSE
    )
  }

  # ============================================================
  # Transform to Gumbel space
  # ============================================================

  mu_vec <- par.temporal[, 1]
  sigma_vec <- par.temporal[, 2]
  xi <- reg_par[5]

  z <-
    1 +
    xi *
    sweep(
      sweep(add_data, 1, mu_vec, "-"),
      1,
      sigma_vec,
      "/"
    )

  if (any(z <= 0, na.rm = TRUE)) {
    stop(
      "Invalid transformed values encountered.",
      call. = FALSE
    )
  }

  IDD.series <- (1 / xi) * log(z)

  # ============================================================
  # Bootstrap setup
  # ============================================================

  sigma_over_xi <- sigma_vec / xi

  reg_par.overall.boot <-
    matrix(
      NA_real_,
      n.boots,
      5
    )

  message("This calculation may take some time.")

  show_pb <- use_progress_bar(progress)

  if (show_pb) {
    pb <- utils::txtProgressBar(
      min = 0,
      max = n.boots,
      style = 3
    )
  }

  # ============================================================
  # Bootstrap loop
  # ============================================================

  for (r in seq_len(n.boots)) {

    rows <- sample.int(
      max_time,
      size = max_time,
      replace = TRUE
    )

    resampled_IDD <-
      IDD.series[rows, , drop = FALSE]

    back.orig <-
      sweep(
        sigma_over_xi *
          (exp(resampled_IDD * xi) - 1),
        1,
        mu_vec,
        "+"
      )

    fit.boot <-
      tryCatch(
        Fit_model(
          temperatures = back.orig,
          model = model
        ),
        error = function(e) NULL
      )

    if (!is.null(fit.boot)) {

      reg_par.overall.boot[r, ] <-
        tryCatch(
          as.numeric(
            Reg_par(fit.boot)
          ),
          error = function(e) rep(NA_real_, 5)
        )
    }

    if (show_pb) {
      utils::setTxtProgressBar(pb, r)
    }
  }

  if (show_pb) {
    close(pb)
  }

  # ============================================================
  # Confidence intervals
  # ============================================================

  CI_lower <-
    apply(
      reg_par.overall.boot,
      2,
      quantile,
      probs = 0.025,
      na.rm = TRUE
    )

  CI_upper <-
    apply(
      reg_par.overall.boot,
      2,
      quantile,
      probs = 0.975,
      na.rm = TRUE
    )

  CI_matrix <-
    rbind(
      CI_lower,
      CI_upper
    )

  rownames(CI_matrix) <-
    c(
      "Lower 95% CI",
      "Upper 95% CI"
    )

  colnames(CI_matrix) <-
    paste0(
      "weighted_",
      c(
        "mu0",
        "mu1",
        "sigma0",
        "sigma1",
        "shape"
      )
    )

  return(CI_matrix)
}

test_that("Reg_parCI returns the correct structure", {
  set.seed(123)

  add_data <- matrix(rnorm(40 * 4), ncol = 4)
  reg_par <- matrix(c(0, 0.01, 1, 0.001, 0.1), nrow = 1)

  result <- Reg_parCI(
    add_data = add_data,
    model = 1,
    reg_par = reg_par,
    n.boots = 100
  )

  expect_type(result, "double")
  expect_identical(dim(result), c(2L, 5L))
  expect_identical(rownames(result), c("Lower 95% CI", "Upper 95% CI"))
  expect_identical(
    colnames(result),
    c(
      "weighted_mu0",
      "weighted_mu1",
      "weighted_sigma0",
      "weighted_sigma1",
      "weighted_shape"
    )
  )
  expect_false(anyNA(result))
})


test_that("Reg_parCI rejects invalid model values", {
  add_data <- matrix(rnorm(40 * 4), ncol = 4)
  reg_par <- matrix(c(0, 0.01, 1, 0.001, 0.1), nrow = 1)

  expect_error(
    Reg_parCI(add_data, model = 5, reg_par, n.boots = 100),
    "`model` must be a single integer between 1 and 4"
  )
  expect_error(
    Reg_parCI(
      add_data,
      model = "two",
      reg_par,
      n.boots = 100
    ),
    "`model` must be a single integer between 1 and 4"
  )
})


test_that("Reg_parCI rejects invalid n.boots", {
  add_data <- matrix(rnorm(40 * 4), ncol = 4)
  reg_par <- matrix(c(0, 0.01, 1, 0.001, 0.1), nrow = 1)

  expect_error(
    Reg_parCI(add_data, model = 1, reg_par, n.boots = 50),
    "`n.boots` must be an integer larger than or equal to 100."
  )
  expect_error(
    Reg_parCI(
      add_data,
      model = 1,
      reg_par,
      n.boots = 99.5
    ),
    "`n.boots` must be an integer larger than or equal to 100."
  )
})


test_that("Reg_parCI rejects non-numeric add_data", {
  reg_par <- matrix(c(0, 0.01, 1, 0.001, 0.1), nrow = 1)

  expect_error(
    Reg_parCI(
      add_data = "not numeric",
      model = 1,
      reg_par = reg_par,
      n.boots = 100
    ),
    "`add_data` must be numeric"
  )
})


test_that("Reg_parCI rejects empty add_data", {
  reg_par <- matrix(c(0, 0.01, 1, 0.001, 0.1), nrow = 1)

  expect_error(
    Reg_parCI(
      add_data = matrix(numeric(0), nrow = 0),
      model = 1,
      reg_par = reg_par,
      n.boots = 100
    ),
    "`add_data` cannot be empty"
  )
})


test_that("Reg_parCI rejects fewer than three sites", {
  add_data <- matrix(rnorm(40 * 2), ncol = 2)
  reg_par <- matrix(c(0, 0.01, 1, 0.001, 0.1), nrow = 1)

  expect_error(
    Reg_parCI(add_data, model = 1, reg_par, n.boots = 100),
    "The number of sites must be larger than 2"
  )
})


test_that("Reg_parCI rejects sites with fewer than ten observations", {
  add_data <- matrix(rnorm(40 * 4), ncol = 4)
  add_data[1:35, 1] <- NA
  reg_par <- matrix(c(0, 0.01, 1, 0.001, 0.1), nrow = 1)

  expect_error(
    Reg_parCI(add_data, model = 1, reg_par,  n.boots = 100),
    "All sites must have at least 10 observations."
  )
})


test_that("Reg_parCI rejects invalid reg_par dimensions", {
  add_data <- matrix(rnorm(40 * 4), ncol = 4)
  reg_par <- matrix(1:4, nrow = 1)

  expect_error(
    Reg_parCI(add_data, model = 1, reg_par, n.boots = 100),
    "`reg_par` must have exactly 1 row and 5 columns"
  )
})


test_that("Reg_parCI rejects non-finite reg_par values", {
  add_data <- matrix(rnorm(40 * 4), ncol = 4)
  reg_par <- matrix(c(0, 0.01, 1, Inf, 0.1), nrow = 1)

  expect_error(
    Reg_parCI(add_data, model = 1, reg_par, n.boots = 100),
    "`reg_par` must contain finite numeric values"
  )
})


test_that("Reg_parCI rejects non-positive time-varying scale parameter", {
  add_data <- matrix(rnorm(40 * 4), ncol = 4)
  reg_par <- matrix(c(0, 0, 1, -0.05, 0.1), nrow = 1)

  expect_error(
    Reg_parCI(add_data, model = 1, reg_par,  n.boots = 100),
    "Time-varying scale parameter became non-positive"
  )
})


test_that("Reg_parCI rejects invalid transformed values", {
  add_data <- matrix(1000, nrow = 40, ncol = 4)
  reg_par <- matrix(c(0, 0, 1, 0, -0.2), nrow = 1)

  expect_error(
    Reg_parCI(add_data, model = 1, reg_par,  n.boots = 100),
    "Invalid transformed values encountered"
  )
})


test_that("Lower confidence limits are smaller than upper confidence limits", {
  set.seed(123)

  add_data <- matrix(rnorm(40 * 4), ncol = 4)
  reg_par <- matrix(c(0, 0.01, 1, 0.001, 0.1), nrow = 1)

  result <- Reg_parCI(
    add_data = add_data,
    model = 1,
    reg_par = reg_par,
    n.boots = 100
  )

  expect_true(all(result[1, ] <= result[2, ]))
})

#' dataset_add
#'
#' @param dataset
#' A numeric matrix with extreme air temperature data from multiple sites.
#' The first column must contain the years, and the remaining columns contain
#' temperature data from each site.
#'
#' @returns
#' A `list` object with the following elements:
#' \describe{
#'   \item{add_data}{A matrix of centered temperature values for each site.}
#'   \item{reg_mean}{Mean temperature values for each site.}
#' }
#'
#' @export
#' @examples
#' Dataset_add(TmaxCPC_SP)

Dataset_add <- function(dataset) {
  dataset.year <- check_dataset(dataset)
  reg_mean <- colMeans(dataset.year, na.rm = TRUE)
  add_data <- scale(dataset.year, center = reg_mean, scale = FALSE)
  list(add_data = add_data, reg_mean = reg_mean)
}

test_that("Dataset_add returns the expected structure", {
  set.seed(123)

  years <- 1980:2009
  temp <- matrix(rnorm(30 * 3), ncol = 3)

  dataset <- cbind(Years = years, temp)

  result <- Dataset_add(dataset)

  expect_type(result, "list")
  expect_named(result, c("add_data", "reg_mean"))

  expect_true(is.matrix(result$add_data))
  expect_type(result$reg_mean, "double")

  expect_identical(
    dim(result$add_data),
    dim(temp)
  )

  expect_length(
    result$reg_mean,
    3
  )
})


test_that("Dataset_add centers each site", {
  set.seed(123)

  years <- 1980:2009
  temp <- matrix(rnorm(30 * 4), ncol = 4)

  dataset <- cbind(Years = years, temp)

  result <- Dataset_add(dataset)

  expect_equal(
    unname(colMeans(result$add_data)),
    rep(0, 4),
    tolerance = 1e-10
  )
})


test_that("Dataset_add preserves missing values", {
  set.seed(123)

  years <- 1980:2009
  temp <- matrix(rnorm(30 * 3), ncol = 3)

  temp[5, 2] <- NA
  temp[10, 3] <- NA

  dataset <- cbind(Years = years, temp)

  result <- Dataset_add(dataset)

  expect_identical(
    sum(is.na(result$add_data)),
    sum(is.na(temp))
  )
})


test_that("Dataset_add returns the correct site means", {
  set.seed(123)

  years <- 1980:2009
  temp <- matrix(rnorm(30 * 3), ncol = 3)

  dataset <- cbind(Years = years, temp)

  result <- Dataset_add(dataset)

  expect_identical(
    unname(result$reg_mean),
    unname(colMeans(temp))
  )
})


test_that("Dataset_add throws error when Years contains NA", {
  years <- 1980:2009
  years[5] <- NA

  temp <- matrix(rnorm(30 * 3), ncol = 3)

  dataset <- cbind(Years = years, temp)

  expect_error(
    Dataset_add(dataset),
    "Column 'Years' cannot have missing data."
  )
})


test_that("Dataset_add throws error when number of sites is too small", {
  years <- 1980:2009
  temp <- matrix(rnorm(30 * 2), ncol = 2)

  dataset <- cbind(Years = years, temp)

  expect_error(
    Dataset_add(dataset),
    "The number of sites should be larger than 2."
  )
})


test_that("Dataset_add throws error when a site has fewer than 10 observations", {
  years <- 1980:2009
  temp <- matrix(rnorm(30 * 3), ncol = 3)

  # Site 1 has only 5 observations
  temp[1:25, 1] <- NA

  dataset <- cbind(Years = years, temp)

  expect_error(
    Dataset_add(dataset),
    "All sites must have at least 10 observations."
  )
})


test_that("Dataset_add throws error for invalid input type", {
  dataset <- "bad data"
  expect_error(
    Dataset_add(dataset),
    "Input 'dataset' must be a matrix or data frame."
  )
})


test_that("Dataset_add works with a data frame input", {
  set.seed(123)

  years <- 1980:2009
  temp <- matrix(rnorm(30 * 3), ncol = 3)

  dataset <- data.frame(
    Years = years,
    Site1 = temp[, 1],
    Site2 = temp[, 2],
    Site3 = temp[, 3]
  )

  result <- Dataset_add(dataset)

  expect_type(result, "list")
  expect_true(is.matrix(result$add_data))
  expect_length(result$reg_mean, 3)
})


test_that("Dataset_add returns add_data with same dimensions as the original site matrix", {
  set.seed(123)

  years <- 1980:2009
  temp <- matrix(rnorm(30 * 5), ncol = 5)

  dataset <- cbind(Years = years, temp)

  result <- Dataset_add(dataset)

  expect_identical(
    dim(result$add_data),
    c(30L, 5L)
  )
})

#' Time-varying parameters of a given GEV model
#'
#' @param temperatures
#' A vector or single-column matrix of air temperature data, either original
#' or centered by subtracting the sample mean.
#'
#' @param model
#' A single integer between 1 and 4 defining the GEV model.
#' May be provided by `Best_model()`.
#'
#' @returns
#' A `data.frame` containing the estimated parameters
#' (`mu0`, `mu1`, `sigma0`, `sigma1`, `shape`,
#' `size`).  If fitting fails for a site, `NA`s are returned for
#' that site.
#'
#' @details
#' The function attempts to fit the model using `ismev::gev.fit()` with
#' the following optimisers in sequence:
#' `Nelder-Mead`, `BFGS`, `CG`, `L-BFGS-B`, `SANN`.
#' The first optimiser that converges is used; if all fail, `NA`s are
#' returned for that site.
#'
#' @importFrom ismev gev.fit
#' @importFrom stats na.omit
#' @importFrom spsUtil quiet
#' @export
#'
#' @examples
#' temperatures <- TmaxCPC_SP$Pixel_1
#' model <- 4
#' Fit_model(temperatures, model)
Fit_model <- function(temperatures, model) {
  if (length(temperatures) == 0) {
    stop("`temperatures` cannot be empty.", call. = FALSE)
  }

  if (!is.numeric(temperatures)) {
    stop("`temperatures` must be a numeric vector or matrix.", call. = FALSE)
  }

  check_model(model) # defined in input_checks.R

  temperatures <- as.matrix(temperatures)
  n.sites <- ncol(temperatures)
  sizes <- integer(n.sites)
  par_mat <- matrix(NA_real_, n.sites, 5)

  for (i in seq_len(n.sites)) {
    local <- na.omit(temperatures[, i, drop = TRUE])
    sizes[i] <- length(local)
    if (sizes[i] == 0L) {
      next
    }

    par_mat[i, ] <- fit_gev_site(local, seq_len(sizes[i]), model_id = model)
  }

  out <- as.data.frame(par_mat)
  out$size <- sizes
  colnames(out) <- c("mu0", "mu1", "sigma0", "sigma1", "shape", "size")
  out
}

# -----------------------------------------------------------------------------
# Internal: call ismev::gev.fit() for a given model and optimiser,
# returning a 5-element numeric parameter vector or NULL on failure.
# Replaces fit_gev_ismev() + fit_gev_alt() + fit_gev().
# -----------------------------------------------------------------------------
#' @noRd
fit_gev_single <- function(local, time, model_id, method) {
  spec <- GEV_MODEL_SPECS[[model_id]]

  fit <- try(
    spsUtil::quiet(
      ismev::gev.fit(
        local,
        ydat = as.matrix(time),
        mul = spec$mul,
        sigl = spec$sigl,
        shl = NULL,
        mulink = identity,
        siglink = identity,
        shlink = identity,
        show = FALSE,
        method = method,
        maxit = 10000L
      )
    ),
    silent = TRUE
  )

  if (inherits(fit, "try-error") || is.null(fit) || is.null(fit$mle)) {
    return(NULL)
  }

  extract_pars(fit, model_id) # defined in best_model.R
}


# -----------------------------------------------------------------------------
# Internal: try each optimiser in turn; return the first success or NA vector.
# -----------------------------------------------------------------------------
#' @noRd
fit_gev_site <- function(local, time, model_id) {
  for (method in OPTIM_METHODS) {
    result <- fit_gev_single(local, time, model_id, method)
    if (!is.null(result)) return(result)
  }
  rep(NA_real_, 5)
}

test_that("Fit_model returns correct output structure for multiple sites", {
  set.seed(123)
  temperatures <- matrix(rnorm(1500), ncol = 15)

  result <- Fit_model(temperatures, model = 2)

  expect_s3_class(result, "data.frame")
  expect_identical(nrow(result), 15L)
  expect_identical(ncol(result), 6L)
  expect_named(result, c("mu0", "mu1", "sigma0", "sigma1", "shape", "size"))
})


test_that("Fit_model works with a vector input", {
  set.seed(123)
  temperatures <- rnorm(100)

  result <- Fit_model(temperatures, model = 1)

  expect_s3_class(result, "data.frame")
  expect_identical(nrow(result), 1L)
  expect_identical(result$size, 100L)
})


test_that("Fit_model accepts models 1 to 4", {
  set.seed(123)
  temperatures <- matrix(rnorm(1500), ncol = 15)

  for (m in 1:4) {
    result <- Fit_model(temperatures, model = m)
    expect_s3_class(result, "data.frame")
    expect_identical(nrow(result), 15L)
  }
})


test_that("Fit_model rejects invalid model values", {
  temperatures <- matrix(rnorm(1500), ncol = 15)

  expect_error(
    Fit_model(temperatures, model = "two"),
    "`model` must be a single integer between 1 and 4."
  )
  expect_error(
    Fit_model(temperatures, model = 7L),
    "`model` must be a single integer between 1 and 4."
  )
  expect_error(
    Fit_model(temperatures, model = 0),
    "`model` must be a single integer between 1 and 4."
  )
  expect_error(
    Fit_model(temperatures, model = 2.5),
    "`model` must be a single integer between 1 and 4."
  )
})


test_that("Fit_model rejects non-numeric temperatures", {
  expect_error(
    Fit_model("not numeric", model = 1),
    "`temperatures` must be a numeric vector or matrix."
  )
})


test_that("Fit_model rejects empty temperatures", {
  expect_error(
    Fit_model(numeric(0), model = 1),
    "`temperatures` cannot be empty."
  )
})


test_that("Fit_model handles missing values within sites", {
  set.seed(123)
  temperatures <- matrix(rnorm(1500), ncol = 15)
  temperatures[1:10, 1] <- NA
  temperatures[20:30, 5] <- NA

  result <- Fit_model(temperatures, model = 1)

  expect_s3_class(result, "data.frame")
  expect_identical(nrow(result), 15L)
  expect_identical(result$size[1], 90L)
  expect_identical(result$size[5], 89L)
})


test_that("Fit_model handles sites with all missing values", {
  temperatures <- matrix(rnorm(1500), ncol = 15)
  temperatures[, 3] <- NA

  result <- Fit_model(temperatures, model = 1)

  expect_identical(result$size[3], 0L)
  expect_true(all(is.na(result[3, 1L:5L])))
})


test_that("Fit_model returns finite sample sizes", {
  set.seed(123)
  temperatures <- matrix(rnorm(1500), ncol = 15)

  result <- Fit_model(temperatures, model = 4)

  expect_true(all(is.finite(result$size)))
  expect_true(all(result$size > 0L))
})


test_that("Fit_model returns numeric parameter columns", {
  set.seed(123)
  temperatures <- matrix(rnorm(1500), ncol = 15)

  result <- Fit_model(temperatures, model = 2)

  expect_type(result$mu0, "double")
  expect_type(result$mu1, "double")
  expect_type(result$sigma0, "double")
  expect_type(result$sigma1, "double")
  expect_type(result$shape, "double")
})


test_that("Fit_model stores correct sample sizes", {
  set.seed(123)
  temperatures <- matrix(rnorm(1500), ncol = 15)
  temperatures[1:10, 1] <- NA
  temperatures[1:20, 2] <- NA

  result <- Fit_model(temperatures, model = 1)

  expect_identical(result$size[1], 90L)
  expect_identical(result$size[2], 80L)
  expect_identical(result$size[3], 100L)
})


test_that("Fit_model works with a single-column matrix", {
  set.seed(123)
  temperatures <- matrix(rnorm(100), ncol = 1)

  result <- Fit_model(temperatures, model = 1)

  expect_s3_class(result, "data.frame")
  expect_identical(nrow(result), 1L)
  expect_identical(result$size, 100L)
})


test_that("Fit_model correctly handles different sample sizes across sites", {
  set.seed(123)
  temperatures <- matrix(rnorm(500), ncol = 5)
  temperatures[1:5, 1] <- NA
  temperatures[1:10, 2] <- NA
  temperatures[1:20, 3] <- NA

  result <- Fit_model(temperatures, model = 2)

  expect_identical(result$size, c(95L, 90L, 80L, 100L, 100L))
})


test_that("Fit_model works with one site", {
  set.seed(123)
  temperatures <- matrix(rnorm(100), ncol = 1)

  result <- Fit_model(temperatures, model = 4)

  expect_identical(nrow(result), 1L)
  expect_identical(result$size, 100L)
})


test_that("Fit_model returns NULL or NA when fitting fails for a site", {
  # Single observation — fitting will fail for all optimisers
  temperatures <- matrix(1, nrow = 1, ncol = 1)

  result <- Fit_model(temperatures, model = 1)

  expect_identical(result$size[1], 1L)
  expect_true(all(is.na(result[1, 1L:5L])))
})


test_that("Fit_model preserves zero coefficients for stationary parameters", {
  set.seed(123)
  temperatures <- matrix(rnorm(1500), ncol = 15)

  r1 <- Fit_model(temperatures, model = 1)
  r2 <- Fit_model(temperatures, model = 2)
  r3 <- Fit_model(temperatures, model = 3)

  # Only check sites where fitting succeeded
  expect_true(all(r1$mu1[!is.na(r1$mu1)] == 0))
  expect_true(all(r1$sigma1[!is.na(r1$sigma1)] == 0))
  expect_true(all(r2$sigma1[!is.na(r2$sigma1)] == 0))
  expect_true(all(r3$mu1[!is.na(r3$mu1)] == 0))
})


test_that("Fit_model handles all models returning length-5 parameter rows", {
  set.seed(123)
  temperatures <- rnorm(100)

  for (m in 1:4) {
    result <- Fit_model(temperatures, model = m)
    expect_identical(ncol(result), 6L)
    expect_length(as.numeric(result[1, 1:5]), 5)
  }
})

#' Geographical coordinates of the `TmaxCPC_SP` series
#'
#'
#' @format ## `lonlat_Tmax`
#' A data.frame with 2 columns and 10 rows.
#'
#' \describe{
#' \item{lon}{Longitude in decinal degrees}
#' \item{lat}{Latitude in decinal degrees}
#' }
#' @source <https://psl.noaa.gov/>
"lonlat_Tmax"

# =============================================================================
# input_checks.R
# Internal validation helpers — not exported.
# Each function throws an informative error or returns invisibly/coerced value.
# =============================================================================

#' @noRd
check_matrix_or_df <- function(x, arg_name) {
  if (!is.matrix(x) && !is.data.frame(x)) {
    stop(
      sprintf("Input '%s' must be a matrix or data frame.", arg_name),
      call. = FALSE
    )
  }
  as.matrix(x)
}

#' @noRd
check_min_site_records <- function(data, min_n = 10L) {
  if (min(colSums(!is.na(data))) < min_n) {
    stop("All sites must have at least 10 observations.", call. = FALSE)
  }
  invisible(NULL)
}

#' @noRd
check_model <- function(model) {
  if (
    !is.numeric(model) ||
      length(model) != 1 ||
      model != as.integer(model) ||
      !(model %in% 1:4)
  ) {
    stop("`model` must be a single integer between 1 and 4.", call. = FALSE)
  }
  invisible(NULL)
}

#' @noRd
check_n.boots <- function(n.boots) {
  if (
    !is.numeric(n.boots) ||
      length(n.boots) != 1 ||
      n.boots != as.integer(n.boots) ||
      n.boots < 100
  ) {
    stop(
      "`n.boots` must be an integer larger than or equal to 100.",
      call. = FALSE
    )
  }
  invisible(NULL)
}

#' @noRd
check_reg_par <- function(reg_par) {
  reg_par <- as.matrix(reg_par)
  if (!is.numeric(reg_par) || !all(is.finite(reg_par))) {
    stop("`reg_par` must contain finite numeric values.", call. = FALSE)
  }
  if (nrow(reg_par) != 1 || ncol(reg_par) != 5) {
    stop("`reg_par` must have exactly 1 row and 5 columns.", call. = FALSE)
  }
  as.numeric(reg_par)
}

#' @noRd
check_site_par <- function(site_par) {
  site_par <- as.matrix(site_par)
  if (!is.numeric(site_par)) {
    stop("`site_par` must be numeric.", call. = FALSE)
  }
  if (nrow(site_par) != 1 || ncol(site_par) != 5) {
    stop("`site_par` must have exactly 1 row and 5 columns.", call. = FALSE)
  }
  if (!all(is.finite(site_par))) {
    stop("`site_par` cannot contain missing or infinite values.", call. = FALSE)
  }
  as.numeric(site_par)
}

#' @noRd
check_atsite_temp <- function(atsite_temp) {
  data_site <- as.matrix(atsite_temp)

  if (ncol(data_site) != 1) {
    stop(
      "`atsite_temp` must be a vector or single-column matrix.",
      call. = FALSE
    )
  }

  if (all(is.na(data_site))) {
    stop("`atsite_temp` contains only missing values.", call. = FALSE)
  }

  if (!is.numeric(data_site)) {
    stop("`atsite_temp` must be numeric.", call. = FALSE)
  }

  if (sum(!is.na(data_site)) < 10) {
    stop("Site must have at least 10 years of records.", call. = FALSE)
  }

  na.omit(as.numeric(data_site))
}

#' @noRd
check_add_data <- function(add_data) {
  if (!is.numeric(add_data)) {
    stop("`add_data` must be numeric.", call. = FALSE)
  }
  add_data <- as.matrix(add_data)
  if (length(add_data) == 0) {
    stop("`add_data` cannot be empty.", call. = FALSE)
  }
  if (ncol(add_data) < 3) {
    stop("The number of sites must be larger than 2.", call. = FALSE)
  }
  if (nrow(add_data) < 10) {
    stop("`add_data` must contain at least 10 observations.", call. = FALSE)
  }
  check_min_site_records(add_data)
  add_data
}

#' @noRd
check_dataset <- function(dataset) {
  dataset <- check_matrix_or_df(dataset, "dataset")
  if (anyNA(dataset[, 1])) {
    stop("Column 'Years' cannot have missing data.", call. = FALSE)
  }
  data_sites <- dataset[, -1, drop = FALSE]
  if (ncol(data_sites) < 3L) {
    stop("The number of sites should be larger than 2.", call. = FALSE)
  }
  check_min_site_records(data_sites)
  data_sites
}

#' @noRd
check_best_model_df <- function(best_model) {
  if (!is.data.frame(best_model)) {
    stop("Input 'best_model' must be a data frame.", call. = FALSE)
  }

  required_cols <- c("mu0", "mu1", "sigma0", "sigma1", "shape", "size")

  if (!all(required_cols %in% names(best_model))) {
    stop(
      "Input 'best_model' must contain the columns: ",
      toString(required_cols),
      ".",
      call. = FALSE
    )
  }

  if (!all(vapply(best_model[required_cols], is.numeric, logical(1)))) {
    stop("All columns in 'best_model' must be numeric.", call. = FALSE)
  }

  if (!all(is.finite(best_model$size)) || any(best_model$size < 0)) {
    stop(
      "The 'size' column must contain finite non-negative values.",
      call. = FALSE
    )
  }

  if (sum(best_model$size) == 0) {
    stop(
      "The sum of the 'size' column must be greater than zero.",
      call. = FALSE
    )
  }

  # Note: NA parameter estimates are valid when fitting fails for a site;
  # only the size column requires finiteness.

  invisible(NULL)
}

#' @noRd
check_quantiles <- function(quantiles) {
  if (!is.numeric(quantiles) || !all(is.finite(quantiles))) {
    stop(
      "`quantiles` must be a numeric vector with no missing values.",
      call. = FALSE
    )
  }
  invisible(NULL)
}

#' @noRd
check_site_temp <- function(site_temp) {
  if (
    !is.numeric(site_temp) ||
      length(site_temp) == 0 ||
      !all(is.finite(site_temp))
  ) {
    stop(
      "`site_temp` must be a non-empty numeric vector or 1-column matrix.",
      call. = FALSE
    )
  }
  invisible(NULL)
}

#' @noRd
check_n.year <- function(n.year, site_temp) {
  if (!is.numeric(n.year) || length(n.year) != 1) {
    stop("`n.year` must be a single numeric value.", call. = FALSE)
  }

  if (n.year != as.integer(n.year)) {
    stop("`n.year` must be a whole number.", call. = FALSE)
  }

  if (n.year < 1 || n.year > length(site_temp)) {
    stop("`n.year` must be between 1 and length(`site_temp`).", call. = FALSE)
  }

  invisible(NULL)
}

#' @noRd
check_prob <- function(prob) {
  if (
    !is.numeric(prob) ||
      !all(is.finite(prob)) ||
      any(prob <= 0) ||
      any(prob >= 1)
  ) {
    stop(
      paste0(
        "`prob` must be a numeric vector with values strictly ",
        "between 0 and 1 and no missing data."
      ),
      call. = FALSE
    )
  }
  invisible(NULL)
}

#' @noRd
check_heterogeneity_data <- function(dataset.add) {
  if (!is.matrix(dataset.add) && !is.data.frame(dataset.add)) {
    stop("'dataset.add' must be a matrix or data frame.", call. = FALSE)
  }
  dataset.add <- as.matrix(dataset.add)
  if (ncol(dataset.add) < 3) {
    stop(
      "The number of sites should be equal to or larger than 3.",
      call. = FALSE
    )
  }
  check_min_site_records(dataset.add)
  dataset.add
}

#' @noRd
check_rho <- function(rho) {
  if (!is.numeric(rho) || length(rho) != 1 || rho >= 1 || rho <= -1) {
    stop(
      "'rho' must be a single numeric value strictly between -1 and 1.",
      call. = FALSE
    )
  }
  invisible(NULL)
}

#' @noRd
check_Ns <- function(Ns) {
  if (!is.numeric(Ns) || length(Ns) != 1 || Ns != as.integer(Ns) || Ns < 100) {
    stop("Ns should be larger than 99.", call. = FALSE)
  }
  invisible(NULL)
}

# =============================================================================
# Progress bar helper
# =============================================================================

#' Show or suppress the progress bar in long-running functions
#'
#' Long-running functions in this package (\code{Site_parCI},
#' \code{Reg_parCI}) display a progress bar by default when the session is
#' interactive. This behaviour can be controlled globally via
#' \code{options(NSTempRFA.progress = FALSE)} or overridden per-call by
#' setting the \code{progress} argument directly on the function.
#'
#' @param progress
#' \code{TRUE} to show, \code{FALSE} to hide, or \code{NULL} (default) to
#' defer to \code{getOption("NSTempRFA.progress")}, which itself falls back
#' to \code{interactive()}.
#'
#' @return Logical scalar.
#' @noRd
use_progress_bar <- function(progress = NULL) {
  if (!is.null(progress)) {
    return(isTRUE(progress))
  }
  opt <- getOption("NSTempRFA.progress", default = NULL)
  if (!is.null(opt)) {
    return(isTRUE(opt))
  }
  interactive()
}

---
title: "CaseStudy"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{CaseStudy}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: bibliography.bib
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
set.seed(23)
options(NSTempRFA.progress = FALSE) # suppress progress bar for session
```

# Introduction
The fundamental concept behind Regional Frequency Analysis (RFA) is to evaluate whether a group of time series from distinct sites can be considered “acceptably homogeneous” [@hosking1997]. 

In its original version, the RFA is valid only when the variable of interest assumes strictly positive values. However, [@martins2022] verified that this technique can be effectively applied to extreme maximum (Tmax) and minimum (Tmin) air temperature series, which may assume both positive and negative values, by adopting the so-called additive approach. 

Although the study by [@martins2022] showed promising results, it did not address the influence of long-term warming trends. This calls for an extension of the RFA framework to incorporate time-dependent, nonstationary statistical properties. 

To overcome this limitation, [@blain2026] proposed the Non-stationary Additive Regional Frequency Analysis, an extension of the additive RFA that integrates nonstationary probabilistic models to represent temporal changes in air temperature frequency distributions at the regional scale. This package was designed to facilitate the application of this new method to extreme Tmax and Tmin data under nonstationary climate conditions.

The NSTempRFA package implements the Non-Stationary Additional Regional Frequency Analysis (NS-Add-RFA) method, designed to assess extreme temperature trends in a regional context. This vignette demonstrates the package's application using 13 annual maximum temperature (Tmax) series (1991-2024) from São Paulo, Brazil. The data were sourced from NOAA's Physical Sciences Laboratory [@noaa_psl], were derived using the block maxima approach (one block per calendar year).

# Getting Started
Install the package from GitHub <https://github.com/gabrielblain/NSTempRFA> (see README) and load it into your R session:

```{r setup}
library(NSTempRFA)
```

## Data
**Table 1** presents the longitude (lon) and latitude (lat), in decimal degrees of the 10 Tmax series.
```{r Table 1}
Table1 <- lonlat_Tmax
Table1
```

**Table 2** displays the Tmax data, in Celsius degrees.
```{r Table 2}
TmaxCPC_SP[, 2:11] <- round(TmaxCPC_SP[, 2:11], 1)
Table2 <- TmaxCPC_SP
Table2
```

## Data Screening

Before analysis, ensure data quality by checking for inconsistencies or outliers. The `Add_Discord()` function (demonstrated below) can help identify problematic series.

```{r Add_Discord}
Add_Discord(TmaxCPC_SP)
```

## Do the 10 Tmax series form an “acceptably homogeneous” group?

We develped the `Add_Heterogeneity()` function to verify if a group of extreme maximum or minimum air temperature series can be deemed as “acceptably homogeneous”. This function calculates the additional heterogeneity  measure (Add_H) as proposed by [@martins2022]. To calculate Add_H we first need to subtract from each data its sample mean. This can be accomplished by applying the `Dataset_add()` function.

```{r Preparing the data}
add.data <- Dataset_add(TmaxCPC_SP)
add.data$add_data
add.data$reg_mean
```

**Then, the `Add_Heterogeneity()` function can be applied as follows:**
```{r Add_H}
set.seed(123)
rho <- 0.49 # The average spatial correlation among the series
Ns <- 500 # Number of Simulation required for calculating
Add_H <- Add_Heterogeneity(dataset.add = add.data$add_data, rho = rho, Ns = Ns)
Add_H
```

**Add_H values equal to or lower than 2 allows us to accept that the group of series can be regarded as acceptably homogeneous.**
Therefore, the Add_H value obtained in this example (~0.90) allowed us to accept such a assumption. In case we had obtained Add_H values larger than 2, we probably could verified which Tmax series were preventing the group to be deemed as acceptably homogeneous. This sort of verification can be performed using the `Add_Discord()` function, which calculates the discordance measure under the additional approach (Add_d). 

```{r Add_d}
Add_Discord(TmaxCPC_SP)
```

The column discord presents the Add_d values for each series. In case of Add_H larger than 2, the series with the largest Add_d values may be removed from the group and the Add_H measure may be re-calculated. It is also worth mentioning that this discordance measure may also be used at the onset of the analysis - before pre-defining the groups - to detect series with gross errors.

## Selecting the best GEV-model

In its current version, the NS-Add-RFA method offers four nonstationary models to describe the whether and how the probabilistic structure of the air temperature series, which forms the homogeneous group, are changing across time. These models are based on the Generalized Extreme Value distribution and are described as follows:

- Model 1: (stationary): The three parameters of the GEV are constant over time.
- Model 2: (homoscedastic): Only the location is allowed to vary over time linearly.
- Model 3: Only the scale parameter is allowed to vary over time linearly.
- Model 4: Both location and scale parameters are allowed to vary over time linearly.

The `Best_model()` function selects the best model among these four candidates using the second-order Akaike Information Criterion.

```{r Best_model}
best.parms <- Best_model(add.data = add.data$add_data)
# The best model is:
as.numeric(best.parms$best) #Model 2
# The at-site parameters are:
best.parms$atsite.models
```

Having defined the best GEV model, the `Reg_par()` function can be applied to specify the parameters of the regional distribution, which is valid for all 13 series.

```{r Reg_par}
regional.parms <- Reg_par(best_model = best.parms$atsite.models)
regional.parms
```

The analysis of the parameters of the regional distribution indicates that the frequency distribution of the Tmax data in the region is experiencing changes in the location parameter. In other words, the distribution is experiencing a change in its central tendency. More specifically, the positive value of *weighted_mu1* indicates that the average the Tmax values increased from 1991 to 2024. 

Additionally, we may also calculate the confidence intervals for the parameters of this regional distribution. This can be accomplished by applying the `Reg_parCI()` as follows:

```{r Reg_parCI}
set.seed(123)
Reg_parCI(
  add_data = add.data$add_data,
  model = best.parms$best,
  reg_par = regional.parms,
  n.boots = 500
)
```

Because the Regional Frequency Analysis uses all series within a homogeneous region to fit the regional distribution, it is expect to reduce parameter estimation uncertainties in respect to the at-site approach. To verify such assumption, we will calculate the confidence interval, through the at-site approach, for the Tmax series of Pixel_1 (see Table 2).

```{r Site_parCI}
set.seed(123)
temperatures <- TmaxCPC_SP$Pixel_1
model <- 2
site_par <- Fit_model(temperatures, model)
if (all(is.finite(as.numeric(site_par[1, 1:5])))) {
  Site_parCI(
    atsite_temp = temperatures,
    model = model,
    site_par = site_par[1, 1:5],
    n.boots = 500
  )
}
```
The differences between the upper and lower 95% confidence intervals (CI) of the regional estimates are lower than those of the at-site approach for all GEV parameter. Additionally,  both upper and lower limits for weighted_mu1 - regional approach (~ 0.079 and ~0.021, respectively) - indicate changes toward warmer conditions. However, for the at-site approach, while the lower limit for mu1  (~ -0.046), indicates a change toward cooler conditions, the upper limit (~0.035) indicates a change towards warmer conditions.

The narrower CI of the parameters of the regional distribution are consistent with the assumption that the NS-Add-RFA method reduces the parameter estimation uncertainties in respect to the at-site approach. Therefore, we may use its statistics to calculate the quantile estimates associated with distinct cumulative probabilities. Additionally, since the best model (model 2) is a non-stationary model, we can also calculate how these quantile estimates changed from the first year (1991) to the last year (2024) of the series. This assessment can be carried out using `Add_RegQuant()` function.

```{r Add_RegQuant}
prob <- c(0.8, 0.85, 0.90, 0.92, 0.93, 0.94, 0.95, 0.97, 0.99)
add.data <- Dataset_add(TmaxCPC_SP)
best.parms <- Best_model(add.data = add.data$add_data)
regional_pars <- Reg_par(best_model = best.parms$atsite.models)
Quantiles_1991 <- Add_RegQuant(
  prob = prob,
  regional_pars = regional_pars,
  site_temp = TmaxCPC_SP$Pixel_1,
  n.year = 1
)
Quantiles_2024 <- Add_RegQuant(
  prob = prob,
  regional_pars = regional_pars,
  site_temp = TmaxCPC_SP$Pixel_1,
  n.year = 34
)
Quantiles_1991
Quantiles_2024
```

The analysis of the quantile estimates for 1991 and 2024 highlights the changes towards warmer conditions that this region in the State of Sao Paulo has experienced over the last 34 years. For example, the quantil estimates associated with a cumulative probability equal to 0.8 raised from ~35.58°C in 1991 to 37.23°C in 2024.

This package also allows an equivalent analysis to that of the `Add_RegQuant()`that indicates whether and how the cumulative probabilities of a given quantile changed over time. This assessment can be carried out using `Add_RegProb()` function.

```{r Add_RegProb}
 quantiles <- c(
   35.58301,
   35.81863,
   36.11727,
   36.26792,
   36.35403,
   36.44996,
   36.55892,
   36.84087,
   37.35040
 )
 add.data <- Dataset_add(TmaxCPC_SP)
 best.parms <- Best_model(add.data = add.data$add_data)
 regional_pars <- Reg_par(best_model = best.parms$atsite.models)
 Probs_1991 <- Add_RegProb(
   quantiles = quantiles,
   regional_pars = regional_pars,
   site_temp = TmaxCPC_SP$Pixel_1,
   n.year = 1
 )
 Probs_2024 <- Add_RegProb(
   quantiles = quantiles,
   regional_pars = regional_pars,
   site_temp = TmaxCPC_SP$Pixel_1,
   n.year = 34
 )
 Probs_1991
 Probs_2024
```
 
# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Add_RegProb.R
\name{Add_RegProb}
\alias{Add_RegProb}
\title{Add_RegProb}
\usage{
Add_RegProb(quantiles, regional_pars, site_temp, n.year)
}
\arguments{
\item{quantiles}{A numeric vector with no missing data containing extreme air temperatures.}

\item{regional_pars}{A 5-column, 1-row \code{data.frame} or \code{matrix} as generated by \code{Reg_par()}.}

\item{site_temp}{A non-empty numeric vector or 1-column matrix of site air temperature data.}

\item{n.year}{A single integer giving the year index for which parameters are calculated.
Must be between 1 and \code{length(site_temp)}.}
}
\value{
A numeric matrix of cumulative probabilities (clamped to [0.0001, 0.9999])
associated with the input quantiles from the regional GEV distribution.
}
\description{
Add_RegProb
}
\examples{
quantiles <- c(37.8, 37.9, 38.0, 38.1, 38.2, 38.5, 39.0)
add.data <- Dataset_add(TmaxCPC_SP)
best.parms <- Best_model(add.data = add.data$add_data)
regional_pars <- Reg_par(best_model = best.parms$atsite.models)
Add_RegProb(
  quantiles     = quantiles,
  regional_pars = regional_pars,
  site_temp     = TmaxCPC_SP$Pixel_1,
  n.year        = 34
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_model.R
\name{Fit_model}
\alias{Fit_model}
\title{Time-varying parameters of a given GEV model}
\usage{
Fit_model(temperatures, model)
}
\arguments{
\item{temperatures}{A vector or single-column matrix of air temperature data, either original
or centered by subtracting the sample mean.}

\item{model}{A single integer between 1 and 4 defining the GEV model.
May be provided by \code{Best_model()}.}
}
\value{
A \code{data.frame} containing the estimated parameters
(\code{mu0}, \code{mu1}, \code{sigma0}, \code{sigma1}, \code{shape},
\code{size}).  If fitting fails for a site, \code{NA}s are returned for
that site.
}
\description{
Time-varying parameters of a given GEV model
}
\details{
The function attempts to fit the model using \code{ismev::gev.fit()} with
the following optimisers in sequence:
\code{Nelder-Mead}, \code{BFGS}, \code{CG}, \code{L-BFGS-B}, \code{SANN}.
The first optimiser that converges is used; if all fail, \code{NA}s are
returned for that site.
}
\examples{
temperatures <- TmaxCPC_SP$Pixel_1
model <- 4
Fit_model(temperatures, model)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Add_Heterogeneity.R
\name{Add_Heterogeneity}
\alias{Add_Heterogeneity}
\title{Add_Heterogeneity}
\usage{
Add_Heterogeneity(dataset.add, rho, Ns)
}
\arguments{
\item{dataset.add}{A numeric matrix of air temperature data as calculated by
\code{Dataset_add()}.}

\item{rho}{A single numeric value describing the average inter-site correlation.
Must be strictly between -1 and 1.}

\item{Ns}{Number of simulated groups of series. Default is 100; at least 500
is recommended.}
}
\value{
Hosking and Wallis' heterogeneity measure using an additive approach,
as proposed in Martins et al. (2022)
\doi{10.1590/1678-4499.20220061}.
Returns \code{NA} if the regional distribution cannot be fitted or if
simulated statistics have zero variance.
}
\description{
Add_Heterogeneity
}
\examples{
add.data <- Dataset_add(TmaxCPC_SP)
Add_Heterogeneity(dataset.add = add.data$add_data, rho = 0.51, Ns = 500)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lonlat_Tmax.R
\docType{data}
\name{lonlat_Tmax}
\alias{lonlat_Tmax}
\title{Geographical coordinates of the \code{TmaxCPC_SP} series}
\format{
\subsection{\code{lonlat_Tmax}}{

A data.frame with 2 columns and 10 rows.

\describe{
\item{lon}{Longitude in decinal degrees}
\item{lat}{Latitude in decinal degrees}
}
}
}
\source{
\url{https://psl.noaa.gov/}
}
\usage{
lonlat_Tmax
}
\description{
Geographical coordinates of the \code{TmaxCPC_SP} series
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/best_model.R
\name{Best_model}
\alias{Best_model}
\title{Time-varying parameters of the best fitted GEV model}
\usage{
Best_model(add.data)
}
\arguments{
\item{add.data}{A numeric matrix of air temperature data as calculated by \code{Dataset_add()}.}
}
\value{
A list with:
\describe{
\item{best}{
Index (1--4) of the model with the lowest total AICc across all sites.
}
\item{atsite.models}{
Data frame containing the estimated parameters
(\code{mu0}, \code{mu1}, \code{sigma0}, \code{sigma1}, \code{shape})
and sample size for each site.
}
}
}
\description{
This function fits four time-varying GEV models (with different assumptions
on non-stationarity in location and/or scale) to each temperature series,
computes the AICc of each model, and selects the best one according to the
lowest total AICc.
}
\details{
Model fitting is performed via \code{ismev::gev.fit()}.  Four nested models
are considered:
\enumerate{
\item Stationary (constant location and scale).
\item Time-varying location only.
\item Time-varying scale only.
\item Time-varying location and scale.
}
For each site the function tries up to five optimisation methods
(\code{Nelder-Mead}, \code{BFGS}, \code{CG}, \code{L-BFGS-B}, \code{SANN})
and uses the first that converges.  Model selection is based on the sum of
site-level AICc values.
}
\examples{
add.data <- Dataset_add(TmaxCPC_SP)
best.parms <- Best_model(add.data = add.data$add_data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Reg_parCI.R
\name{Reg_parCI}
\alias{Reg_parCI}
\title{Confidence intervals for regional GEV parameters}
\usage{
Reg_parCI(add_data, model, reg_par, n.boots = 999, progress = NULL)
}
\arguments{
\item{add_data}{A numeric matrix of air temperature data as calculated by \code{Dataset_add()}.}

\item{model}{A single integer number from 1 to 4 defining the GEV model.
May be provided by \code{Best_model()}.}

\item{reg_par}{A 5-column and 1-row data.frame or matrix as generated by \code{Reg_par()}.
\itemize{
\item 1st column is the mu0 parameter,
\item 2nd is the mu1 parameter,
\item 3rd is the sigma0 parameter,
\item 4th is the sigma1 parameter,
\item 5th is the shape parameter.
}}

\item{n.boots}{A single number describing the number of bootstrap replicates.
Whenever possible, \code{n.boots} should be set to 999 (default),
as suggested by Burn (2003) <10.1623/hysj.48.1.25.43485>
and O'Brien and Burn (2014) <10.1016/j.jhydrol.2014.09.041>.}

\item{progress}{Logical scalar controlling whether a progress bar is displayed.
\code{TRUE} forces the bar on, \code{FALSE} forces it off, and
\code{NULL} (default) defers to \code{getOption("NSTempRFA.progress")},
which itself falls back to \code{interactive()}. Set
\code{options(NSTempRFA.progress = FALSE)} to suppress the bar globally.}
}
\value{
A matrix containing the 95\% confidence intervals (lower and upper bounds)
of the time-varying parameter estimates. The spatial dependence between
sites is preserved.
}
\description{
Confidence intervals for regional GEV parameters
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
add.data <- Dataset_add(TmaxCPC_SP)
best.parms <- Best_model(add.data = add.data$add_data)
regional.parms <- Reg_par(best_model = best.parms$atsite.models)

Reg_parCI(
  add_data  = add.data$add_data,
  model     = best.parms$best,
  reg_par   = regional.parms,
  n.boots   = 100
)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_add.R
\name{Dataset_add}
\alias{Dataset_add}
\title{dataset_add}
\usage{
Dataset_add(dataset)
}
\arguments{
\item{dataset}{A numeric matrix with extreme air temperature data from multiple sites.
The first column must contain the years, and the remaining columns contain
temperature data from each site.}
}
\value{
A \code{list} object with the following elements:
\describe{
\item{add_data}{A matrix of centered temperature values for each site.}
\item{reg_mean}{Mean temperature values for each site.}
}
}
\description{
dataset_add
}
\examples{
Dataset_add(TmaxCPC_SP)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Site_parCI.R
\name{Site_parCI}
\alias{Site_parCI}
\title{Confidence intervals for site-specific GEV parameters}
\usage{
Site_parCI(atsite_temp, model, site_par, n.boots = 999, progress = NULL)
}
\arguments{
\item{atsite_temp}{A numeric vector or single-column matrix of air temperature data.}

\item{model}{A single integer from 1 to 4 defining the GEV model.}

\item{site_par}{A 1-row, 5-column data.frame or matrix generated by Fit_model().}

\item{n.boots}{Number of bootstrap replicates. Values smaller than 100 are not allowed.
Defaults to \code{999}.}

\item{progress}{Logical scalar controlling whether a progress bar is displayed.
\code{TRUE} forces the bar on, \code{FALSE} forces it off, and
\code{NULL} (default) defers to \code{getOption("NSTempRFA.progress")},
which itself falls back to \code{interactive()}. Set
\code{options(NSTempRFA.progress = FALSE)} to suppress the bar globally.}
}
\value{
A matrix containing the lower and upper 95\% confidence limits of
the estimated parameters.
}
\description{
Confidence intervals for site-specific GEV parameters
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
temperatures <- TmaxCPC_SP$Pixel_1
model <- 2
site_par <- Fit_model(temperatures, model)

Site_parCI(
  atsite_temp = temperatures,
  model       = model,
  site_par    = site_par[1, 1:5],
  n.boots     = 100
)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reg_par.R
\name{Reg_par}
\alias{Reg_par}
\title{Estimate the parameters of the regional distribution}
\usage{
Reg_par(best_model)
}
\arguments{
\item{best_model}{A 6-column \code{data.frame} as generated by \code{Best_model()}.
\itemize{
\item Columns 1--5: \code{mu0}, \code{mu1}, \code{sigma0},
\code{sigma1}, \code{shape}.
\item Column 6: \code{size} (sample size per site, used as weight).
}}
}
\value{
A one-row \code{data.frame} with the size-weighted regional parameters of a
time-varying generalized extreme value distribution.
}
\description{
Estimate the parameters of the regional distribution
}
\examples{
add.data <- Dataset_add(TmaxCPC_SP)
best.parms <- Best_model(add.data = add.data$add_data)
Reg_par(best_model = best.parms$atsite.models)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Add_Discord.R
\name{Add_Discord}
\alias{Add_Discord}
\title{Discordance measures calculated from the additive approach}
\usage{
Add_Discord(dataset)
}
\arguments{
\item{dataset}{A numeric matrix with extreme air temperature data from multiple sites.
The first column must contain the years, and the remaining columns contain
temperature data from each site.}
}
\value{
A \code{data.frame} with 8 columns:
\describe{
\item{Local}{Site identifier.}
\item{SampleSize}{Number of observations for each site.}
\item{l_1}{First L-moment (mean).}
\item{l_2}{Second L-moment (L-scale).}
\item{t_3}{L-skewness ratio.}
\item{t_4}{L-kurtosis ratio.}
\item{t_5}{Fifth-order L-moment ratio.}
\item{discord}{Original discordance statistic indicating potential outlier status.}
}
}
\description{
This function calculates the Hosking and Wallis' discordance measure (discord) and
the robust discordance measure (Neykov et al. 2007) \url{doi:10.1029/2006WR005322} using
the additive approach proposed in Martins et al. (2022) \url{doi:10.1590/1678-4499.20220061}.
}
\details{
The discordance measures identify sites that are potentially discordant
within a regional frequency analysis framework for extreme air temperature series.
}
\examples{
Add_Discord(TmaxCPC_SP)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Add_RegQuant.R
\name{Add_RegQuant}
\alias{Add_RegQuant}
\title{Regional Quantile Estimation for a Site Using Time-Varying GEV Parameters}
\usage{
Add_RegQuant(prob, regional_pars, site_temp, n.year)
}
\arguments{
\item{prob}{A numeric vector of probabilities strictly between 0 and 1, with no
missing values.}

\item{regional_pars}{A 1-row, 5-column \code{matrix} or \code{data.frame} containing regional
GEV parameters: location intercept, location slope, scale intercept,
scale slope, and shape.}

\item{site_temp}{A numeric vector or 1-column matrix of site air temperature data.}

\item{n.year}{A single integer indicating the year index for which quantiles are
calculated. Must be between 1 and \code{length(site_temp)}.}
}
\value{
A named numeric vector of adjusted regional quantiles for the specified
year. Names are of the form \code{"Q90"}, `"Q95"``, etc.
}
\description{
Computes quantiles for a given site based on regional GEV parameters and
a time-varying location and scale model. Estimated quantiles are adjusted
by the site's mean temperature.
}
\examples{
prob <- c(0.90, 0.92, 0.93, 0.94, 0.95, 0.97, 0.99)
add.data <- Dataset_add(TmaxCPC_SP)
best.parms <- Best_model(add.data = add.data$add_data)
regional_pars <- Reg_par(best_model = best.parms$atsite.models)
Add_RegQuant(
  prob          = prob,
  regional_pars = regional_pars,
  site_temp     = TmaxCPC_SP$Pixel_1,
  n.year        = 34
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TmaxCPC_SP.R
\docType{data}
\name{TmaxCPC_SP}
\alias{TmaxCPC_SP}
\title{Extreme air temperature data}
\format{
\subsection{\code{TmaxCPC_SP}}{

A data.frame with 11 columns and 34 rows.

\describe{
\item{Year}{Year}
\item{Pixel_1 to Pixel_10}{daily extremes for air temperature}
}
}
}
\source{
\url{https://psl.noaa.gov/}
}
\usage{
TmaxCPC_SP
}
\description{
Daily extremes for air temperature data sampled from
the block maxima approach. A block corresponds to one year.
The data belongs to The Climate Prediction Center - NOAA.
location of Ribeirao Preto, State of Sao Paulo in Brazil.
}
\keyword{datasets}
